import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { Clock, ClockTask } from '../../lab/util/Clock.js';
import { CommonControls } from './CommonControls.js';
import { DiffEqSolverSubject } from '../../lab/model/DiffEqSolverSubject.js';
import { DisplayClock } from '../../lab/view/DisplayClock.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyBarGraph } from '../../lab/graph/EnergyBarGraph.js';
import { LabelControl } from '../../lab/controls/LabelControl.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimRunner } from '../../lab/app/SimRunner.js';
import { SimView } from '../../lab/view/SimView.js';
import { StandardGraph1 } from './StandardGraph1.js';
import { TabLayout } from './TabLayout.js';
import { TimeGraph1 } from './TimeGraph1.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { VarsHistory } from '../../lab/graph/VarsHistory.js';
import { FunctionVariable } from '../../lab/model/FunctionVariable.js';
import { GenericMemo } from '../../lab/util/Memo.js';
import { DisplayText } from '../../lab/view/DisplayText.js';
export class AbstractApp0 extends AbstractSubject {
    constructor(elem_ids, simRect, sim, advance, eventHandler, energySystem, opt_name) {
        super(opt_name || 'APP');
        this.energyGraph = null;
        this.showEnergyParam = null;
        this.simRect = simRect;
        this.layout = this.makeLayout(elem_ids);
        this.terminal = this.layout.getTerminal();
        const simCanvas = this.layout.getSimCanvas();
        this.sim = sim;
        this.sim.setTerminal(this.terminal);
        this.advance = advance;
        this.simList = sim.getSimList();
        this.varsList = sim.getVarsList();
        this.simCtrl = new SimController(simCanvas, eventHandler);
        this.simView = new SimView('SIM_VIEW', this.simRect);
        simCanvas.addView(this.simView);
        this.displayList = this.simView.getDisplayList();
        this.statusView = new SimView('STATUS_VIEW', new DoubleRect(-10, -10, 10, 10));
        simCanvas.addView(this.statusView);
        this.axes = CommonControls.makeAxes(this.simView, true);
        this.simRun = new SimRunner(this.advance);
        this.simRun.addCanvas(simCanvas);
        this.simRun.addErrorObserver(this.simCtrl);
        this.clock = this.simRun.getClock();
        if (energySystem != null) {
            this.energyGraph = new EnergyBarGraph(energySystem);
            this.showEnergyParam = CommonControls.makeShowEnergyParam(this.energyGraph, this.statusView, this);
        }
        this.displayClock = new DisplayClock(() => sim.getTime(), () => this.clock.getRealTime(), 2, 2);
        this.displayClock.setPosition(new Vector(8, 4));
        this.showClockParam = CommonControls.makeShowClockParam(this.displayClock, this.statusView, this);
        const panzoom = CommonControls.makePanZoomControls(this.simView, true, () => this.simView.setSimRect(this.simRect));
        this.layout.getSimDiv().appendChild(panzoom);
        this.panZoomParam = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.panZoomParam.setValue(false);
        this.diffEqSolver = new DiffEqSolverSubject(sim, energySystem, advance);
        this.graph = new StandardGraph1(sim.getVarsList(), this.layout.getGraphCanvas(), this.layout.getGraphControls(), this.layout.getGraphDiv(), this.simRun);
        this.graph.line.setDrawingMode("lines");
        this.timeGraph = this.makeTimeGraph();
        this.easyScript;
        this.terminal.setAfterEval(() => this.sim.modifyObjects());
    }
    ;
    toString() {
        return ', sim: ' + this.sim.toStringShort()
            + ', simList: ' + this.simList.toStringShort()
            + ', simCtrl: ' + this.simCtrl.toStringShort()
            + ', advance: ' + this.advance
            + ', simRect: ' + this.simRect
            + ', simView: ' + this.simView.toStringShort()
            + ', statusView: ' + this.statusView.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', simRun: ' + this.simRun.toStringShort()
            + ', clock: ' + this.clock.toStringShort()
            + ', energyGraph: ' + (this.energyGraph == null ? 'null' :
            this.energyGraph.toStringShort())
            + ', displayClock: ' + this.displayClock.toStringShort()
            + ', graph: ' + this.graph.toStringShort()
            + ', timeGraph: ' + this.timeGraph.toStringShort()
            + ', layout: ' + this.layout.toStringShort()
            + ', easyScript: ' + this.easyScript.toStringShort()
            + ', terminal: ' + this.terminal
            + super.toString();
    }
    ;
    makeLayout(elem_ids) {
        const canvasWidth = 800;
        const canvasHeight = Math.round(canvasWidth * this.simRect.getHeight() / this.simRect.getWidth());
        return new TabLayout(elem_ids, canvasWidth, canvasHeight, true);
    }
    ;
    addControl(control) {
        return this.layout.addControl(control);
    }
    ;
    addPlaybackControls() {
        this.addControl(CommonControls.makePlaybackControls(this.simRun));
    }
    ;
    addStandardControls() {
        if (this.showEnergyParam != null) {
            this.addControl(new CheckBoxControl(this.showEnergyParam));
        }
        this.addControl(new CheckBoxControl(this.showClockParam));
        this.addControl(new CheckBoxControl(this.panZoomParam));
        let pn = this.simRun.getParameterNumber(SimRunner.en.TIME_STEP);
        this.addControl(new NumericControl(pn));
        pn = this.simRun.getClock().getParameterNumber(Clock.en.TIME_RATE);
        this.addControl(new NumericControl(pn));
        const ps = this.diffEqSolver.getParameterString(DiffEqSolverSubject.en.DIFF_EQ_SOLVER);
        this.addControl(new ChoiceControl(ps));
        const bm = CommonControls.makeBackgroundMenu(this.layout.getSimCanvas());
        this.addControl(bm);
        if (0 == 1 && Util.DEBUG) {
            this.addControl(new LabelControl('compiled ' + Util.COMPILE_TIME));
        }
    }
    ;
    defineNames(myName) {
        this.simRun.setAppName(myName);
        this.terminal.addToVars(myName);
        this.terminal.addRegex('advance|axes|clock|diffEqSolver|displayClock|energyGraph'
            + '|graph|layout|sim|simCtrl|simList|simRect|simRun|simView|statusView'
            + '|timeGraph|easyScript|terminal|varsList|displayList', myName + '.');
        this.terminal.addRegex('simCanvas', myName + '.layout.');
    }
    ;
    makeEasyScript(opt_dependent) {
        let dependent = [this.varsList];
        if (opt_dependent) {
            dependent = dependent.concat(opt_dependent);
        }
        this.easyScript = CommonControls.makeEasyScript(this.getSubjects(), dependent, this.simRun, this.terminal);
    }
    ;
    getSubjects() {
        const subjects = [
            this,
            this.sim,
            this.diffEqSolver,
            this.simRun,
            this.clock,
            this.simView,
            this.statusView,
            this.varsList
        ];
        return subjects.concat(this.layout.getSubjects(), this.graph.getSubjects(), this.timeGraph.getSubjects());
    }
    ;
    addURLScriptButton() {
        this.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
    static loadClass() {
        var f = FunctionVariable.toString;
        f = VarsHistory.toString;
        f = GenericMemo.toString;
        f = DisplayText.toString;
        f = ClockTask.toString;
    }
    ;
    setup() {
        this.clock.resume();
        this.terminal.parseURL();
        this.sim.saveInitialState();
        this.sim.modifyObjects();
        this.simRun.memorize();
    }
    ;
    start() {
        this.simRun.startFiring();
    }
    ;
}
export class AbstractApp extends AbstractApp0 {
    constructor(elem_ids, simRect, sim, advance, eventHandler, energySystem, opt_name) {
        super(elem_ids, simRect, sim, advance, eventHandler, energySystem, opt_name);
    }
    ;
    makeTimeGraph() {
        return new TimeGraph1(this.sim.getVarsList(), this.layout.getTimeGraphCanvas(), this.layout.getTimeGraphControls(), this.layout.getTimeGraphDiv(), this.simRun);
    }
}
;
Util.defineGlobal('sims$common$AbstractApp', AbstractApp);
