import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { AutoScale } from '../../lab/graph/AutoScale.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CommonControls } from './CommonControls.js';
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericEvent } from '../../lab/util/Observe.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
export class CompareGraph extends AbstractSubject {
    constructor(line1, line2, graphCanvas, div_controls, div_graph, simRun) {
        super('GRAPH_LAYOUT');
        this.controls_ = [];
        this.line1 = line1;
        this.line2 = line2;
        this.canvas = graphCanvas;
        simRun.addCanvas(graphCanvas);
        this.view = new SimView('graph', new DoubleRect(0, 0, 1, 1));
        this.view.setHorizAlign("FULL");
        this.view.setVerticalAlign("FULL");
        this.view.addMemo(line1);
        this.view.addMemo(line2);
        graphCanvas.addView(this.view);
        this.axes = CommonControls.makeAxes(this.view);
        const updateAxes = (evt) => {
            if (evt.nameEquals(GraphLine.en.X_VARIABLE)) {
                this.axes.setHorizName(this.line1.getXVarName());
            }
            if (evt.nameEquals(GraphLine.en.Y_VARIABLE)) {
                this.axes.setVerticalName(this.line1.getYVarName());
            }
        };
        new GenericObserver(line1, updateAxes, 'update axes names');
        updateAxes(new GenericEvent(line1, GraphLine.i18n.X_VARIABLE));
        this.autoScale = new AutoScale('COMPARE_GRAPH_AUTO_SCALE', line1, this.view);
        this.autoScale.addGraphLine(line2);
        this.autoScale.extraMargin = 0.05;
        this.displayGraph = new DisplayGraph(line1);
        this.displayGraph.addGraphLine(line2);
        this.displayGraph.setScreenRect(this.view.getScreenRect());
        this.displayGraph.setUseBuffer(true);
        this.view.getDisplayList().prepend(this.displayGraph);
        new GenericObserver(this.view, evt => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayGraph.setScreenRect(this.view.getScreenRect());
            }
        }, 'resize DisplayGraph');
        this.div_controls = div_controls;
        this.addControl(CommonControls.makePlaybackControls(simRun));
        let pn = line1.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'Y:'));
        pn = line1.getParameterNumber(GraphLine.en.X_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'X:'));
        const bc = new ButtonControl(GraphLine.i18n.CLEAR_GRAPH, () => {
            line1.reset();
            line2.reset();
            this.autoScale.reset();
        });
        this.addControl(bc);
        const ps = line1.getParameterString(GraphLine.en.DRAWING_MODE);
        this.addControl(new ChoiceControl(ps));
        new GenericObserver(line1, (_evt) => line2.setDrawingMode(line1.getDrawingMode()), 'match drawing mode on GraphLine');
        this.graphCtrl = new SimController(graphCanvas, null, { alt: false, control: false, meta: false, shift: false });
        const panzoom = CommonControls.makePanZoomControls(this.view, true, () => this.autoScale.setActive(true));
        div_graph.appendChild(panzoom);
        const pb = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.addControl(new CheckBoxControl(pb));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', line1: ' + this.line1.toStringShort()
            + ', line2: ' + this.line2.toStringShort()
            + ', canvas: ' + this.canvas.toStringShort()
            + ', view: ' + this.view.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', autoScale: ' + this.autoScale.toStringShort()
            + ', displayGraph: ' + this.displayGraph.toStringShort()
            + ', graphCtrl: ' + this.graphCtrl.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'CompareGraph';
    }
    ;
    getSubjects() {
        return [this, this.line1, this.line2, this.view, this.autoScale];
    }
    ;
    addControl(control) {
        const element = control.getElement();
        element.style.display = 'block';
        this.div_controls.appendChild(element);
        this.controls_.push(control);
        return control;
    }
    ;
}
Util.defineGlobal('sims$common$CompareGraph', CompareGraph);
