import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { LabCanvas } from '../../lab/view/LabCanvas.js';
import { ParameterBoolean, ParameterNumber, ParameterString } from '../../lab/util/Observe.js';
import { Terminal } from '../../lab/util/Terminal.js';
import { Util } from '../../lab/util/Util.js';
;
export class TabLayout extends AbstractSubject {
    constructor(elem_ids, canvasWidth = 800, canvasHeight = 800, opt_terminal = true) {
        super('TAB_LAYOUT');
        this.limitSize_ = true;
        this.simWidth_ = 1;
        this.graphWidth_ = 1;
        this.timeGraphWidth_ = 1;
        this.debug_layout = false;
        this.controls_ = [];
        this.show_term_cb = null;
        Util.setImagesDir(elem_ids.images_dir);
        this.tab_list = Util.getElementById(elem_ids.tab_list);
        this.layout_ = this.getSelectedTab();
        this.terminalEnabled_ = opt_terminal;
        if (this.layout_ == '') {
            this.layout_ = "sim";
            this.setSelectedTab(this.layout_);
        }
        this.tab_list.addEventListener('click', e => {
            const target = e.target;
            if (target === undefined) {
                throw 'event target is undefined';
            }
            if (target == null || target.tagName != 'LI') {
                return;
            }
            if (target.className.indexOf('selected') > -1) {
                return;
            }
            this.setLayoutFromTab(target.className);
        });
        window.addEventListener('resize', () => this.redoLayout());
        const orient = window.screen.orientation;
        if (orient !== undefined) {
            orient.addEventListener('orientationchange', () => this.redoLayout());
        }
        else {
            window.addEventListener('orientationchange', () => this.redoLayout());
        }
        const term_output = Util.maybeElementById(elem_ids.term_output);
        this.term_input = Util.maybeElementById(elem_ids.term_input);
        this.terminal = new Terminal(this.term_input, term_output);
        Terminal.stdRegex(this.terminal);
        this.div_contain = Util.getElementById(elem_ids.container);
        if (this.debug_layout) {
            this.div_contain.style.border = 'dashed 1px red';
        }
        this.div_sim = Util.getElementById(elem_ids.sim_applet);
        this.div_sim.style.position = 'relative';
        const canvas = document.createElement('canvas');
        canvas.tabIndex = 0;
        this.simCanvas = new LabCanvas(canvas, 'SIM_CANVAS');
        this.simCanvas.setSize(canvasWidth, canvasHeight);
        this.div_sim.appendChild(this.simCanvas.getCanvas());
        this.show_sim_cb = Util.getElementById(elem_ids.show_sim);
        const p = this.show_sim_cb.parentElement;
        if (p == null || p.tagName != 'LABEL') {
            throw '';
        }
        this.show_sim_label = p;
        this.show_sim_cb.addEventListener('click', _e => { this.showSim(this.show_sim_cb.checked); });
        this.div_graph = Util.getElementById(elem_ids.div_graph);
        this.div_graph.style.position = 'relative';
        const canvas2 = document.createElement('canvas');
        this.graphCanvas = new LabCanvas(canvas2, 'GRAPH_CANVAS');
        canvasWidth = Math.max(canvasWidth, canvasHeight);
        this.graphCanvas.setSize(canvasWidth, canvasWidth);
        this.div_graph.appendChild(canvas2);
        this.div_graph_controls =
            Util.getElementById(elem_ids.graph_controls);
        if (this.debug_layout) {
            this.div_graph_controls.style.border = 'dashed 1px green';
        }
        this.div_sim_controls =
            Util.getElementById(elem_ids.sim_controls);
        this.div_sim_controls.style.marginLeft = '10px';
        if (this.debug_layout) {
            this.div_sim_controls.style.border = 'dashed 1px green';
        }
        this.div_term =
            Util.getElementById(elem_ids.div_terminal);
        this.div_term.style.display = 'none';
        if (this.debug_layout) {
            this.div_term.style.border = 'dashed 1px green';
        }
        const label_term = Util.getElementById(elem_ids.label_terminal);
        if (!this.terminalEnabled_) {
            label_term.style.display = 'none';
        }
        else {
            label_term.style.display = 'inline';
            this.show_term_cb =
                Util.getElementById(elem_ids.show_terminal);
            this.show_term_cb.addEventListener('click', _e => {
                if (this.show_term_cb !== null)
                    this.showTerminal(this.show_term_cb.checked);
            });
        }
        this.div_time_graph =
            Util.getElementById(elem_ids.div_time_graph);
        this.div_time_graph.style.position = 'relative';
        const canvas3 = document.createElement('canvas');
        this.timeGraphCanvas = new LabCanvas(canvas3, 'TIME_GRAPH_CANVAS');
        this.timeGraphCanvas.setSize(canvasWidth, canvasWidth);
        this.div_time_graph.appendChild(canvas3);
        this.div_time_graph_controls =
            Util.getElementById(elem_ids.time_graph_controls);
        if (this.debug_layout) {
            this.div_time_graph_controls.style.border = 'dashed 1px green';
        }
        this.redoLayout();
        this.addParameter(new ParameterNumber(this, TabLayout.en.SIM_WIDTH, TabLayout.i18n.SIM_WIDTH, () => this.getSimWidth(), a => this.setSimWidth(a)));
        this.addParameter(new ParameterNumber(this, TabLayout.en.GRAPH_WIDTH, TabLayout.i18n.GRAPH_WIDTH, () => this.getGraphWidth(), a => this.setGraphWidth(a)));
        this.addParameter(new ParameterNumber(this, TabLayout.en.TIME_GRAPH_WIDTH, TabLayout.i18n.TIME_GRAPH_WIDTH, () => this.getTimeGraphWidth(), a => this.setTimeGraphWidth(a)));
        this.addParameter(new ParameterString(this, TabLayout.en.LAYOUT, TabLayout.i18n.LAYOUT, () => this.getLayout(), a => this.setLayout(a), TabLayout.getLayoutValues(), TabLayout.getLayoutValues()));
        if (this.terminalEnabled_) {
            this.addParameter(new ParameterBoolean(this, TabLayout.en.SHOW_TERMINAL, TabLayout.i18n.SHOW_TERMINAL, () => this.show_term_cb != null ? this.show_term_cb.checked : false, a => this.showTerminal(a)));
        }
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', layout_: "' + this.layout_ + '"'
            + ', simWidth_: ' + Util.NF(this.simWidth_)
            + ', graphWidth_: ' + Util.NF(this.graphWidth_)
            + ', timeGraphWidth_: ' + Util.NF(this.timeGraphWidth_)
            + ', simCanvas: ' + this.simCanvas.toStringShort()
            + ', graphCanvas: ' + this.graphCanvas.toStringShort()
            + ', timeGraphCanvas: ' + this.timeGraphCanvas.toStringShort()
            + ', terminal: ' + this.terminal
            + ', controls_: ['
            + this.controls_.map(a => a.toStringShort())
            + ']'
            + super.toString();
    }
    ;
    getClassName() {
        return 'TabLayout';
    }
    ;
    static getLayoutValues() {
        return ["sim",
            "graph",
            "sim+graph",
            "time_graph",
            "sim+time_graph",
            "multi_graph",
            "sim+multi_graph"
        ];
    }
    ;
    addControl(control, opt_add = true) {
        if (opt_add) {
            const element = control.getElement();
            element.style.display = 'block';
            this.div_sim_controls.appendChild(element);
        }
        this.controls_.push(control);
        return control;
    }
    ;
    alignCanvasControls(canvas, controls, canvas2) {
        canvas.style.display = 'block';
        controls.style.display = 'inline-block';
        controls.style.columnCount = '1';
        controls.style.width = 'auto';
        let ctrl_width = controls.getBoundingClientRect().width;
        ctrl_width = ctrl_width > 150 ? ctrl_width : 300;
        let cvs_width = canvas.offsetWidth || canvas.getBoundingClientRect().width;
        if (canvas2 != null) {
            canvas2.style.display = 'block';
            cvs_width += canvas2.offsetWidth || canvas2.getBoundingClientRect().width;
        }
        const contain_width = this.div_contain.offsetWidth ||
            this.div_contain.getBoundingClientRect().width;
        const avail_width = contain_width - cvs_width - 2;
        if (avail_width < ctrl_width && contain_width > 2 * ctrl_width) {
            controls.style.width = '100%';
            controls.style.columnCount = '2';
        }
    }
    ;
    getGraphCanvas() {
        return this.graphCanvas;
    }
    ;
    getGraphControls() {
        return this.div_graph_controls;
    }
    ;
    getGraphDiv() {
        return this.div_graph;
    }
    ;
    getGraphWidth() {
        return this.graphWidth_;
    }
    ;
    getLayout() {
        return this.layout_;
    }
    ;
    getSelectedTab() {
        let tab = undefined;
        this.tab_list.childNodes.forEach(n => {
            if (n.nodeType == Node.ELEMENT_NODE) {
                const elem = n;
                if (elem.tagName == 'LI') {
                    if (elem.className.match(/.*selected/) != null) {
                        tab = elem;
                    }
                }
            }
        });
        if (tab === undefined) {
            return '';
        }
        const tab2 = tab;
        return tab2.className.replace(/[ ]*selected/, '');
    }
    ;
    getSimCanvas() {
        return this.simCanvas;
    }
    ;
    getSimControls() {
        return this.div_sim_controls;
    }
    ;
    getSimDiv() {
        return this.div_sim;
    }
    ;
    getSimWidth() {
        return this.simWidth_;
    }
    ;
    getSubjects() {
        return [this, this.simCanvas, this.graphCanvas, this.timeGraphCanvas];
    }
    ;
    getTerminal() {
        return this.terminal;
    }
    ;
    getTimeGraphCanvas() {
        return this.timeGraphCanvas;
    }
    ;
    getTimeGraphControls() {
        return this.div_time_graph_controls;
    }
    ;
    getTimeGraphDiv() {
        return this.div_time_graph;
    }
    ;
    getTimeGraphWidth() {
        return this.timeGraphWidth_;
    }
    ;
    redoLayout() {
        const view_width = Util.getViewportSize()[0];
        this.div_sim.style.float = 'left';
        this.div_graph.style.float = 'left';
        this.div_time_graph.style.float = 'left';
        switch (this.layout_) {
            case '':
            case "sim":
                this.div_graph.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_time_graph.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.setDisplaySize(0.95 * this.simWidth_, this.div_graph);
                this.alignCanvasControls(this.div_sim, this.div_sim_controls);
                this.show_sim_label.style.display = 'none';
                break;
            case "graph":
                this.div_sim.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.div_time_graph.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.setDisplaySize(0.95 * this.graphWidth_, this.div_graph);
                this.alignCanvasControls(this.div_graph, this.div_graph_controls);
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+graph":
                this.div_time_graph.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                if (view_width > 600) {
                    this.setDisplaySize(0.49, this.div_graph);
                }
                else {
                    this.setDisplaySize(0.95 * this.graphWidth_, this.div_graph);
                }
                this.alignCanvasControls(this.div_graph, this.div_graph_controls, this.div_sim);
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            case "time_graph":
                this.div_graph.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_sim.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.setDisplaySize(0.95 * this.timeGraphWidth_, this.div_time_graph);
                this.alignCanvasControls(this.div_time_graph, this.div_time_graph_controls);
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+time_graph":
                this.div_graph.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                if (view_width > 600) {
                    this.setDisplaySize(0.49, this.div_time_graph);
                }
                else {
                    this.setDisplaySize(0.95 * this.timeGraphWidth_, this.div_time_graph);
                }
                this.alignCanvasControls(this.div_time_graph, this.div_time_graph_controls, this.div_sim);
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            case "multi_graph":
                this.div_graph.style.display = 'block';
                this.div_time_graph.style.display = 'block';
                this.div_sim.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.setDisplaySize(0.49, this.div_graph);
                this.setDisplaySize(0.49, this.div_time_graph);
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+multi_graph":
                this.div_graph.style.display = 'block';
                this.div_time_graph.style.display = 'block';
                this.div_sim.style.display = 'block';
                this.div_sim_controls.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.setDisplaySize(0.49, this.div_graph);
                this.setDisplaySize(0.49, this.div_time_graph);
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            default:
                throw 'redoLayout: no such layout "' + this.layout_ + '"';
        }
    }
    ;
    setDisplaySize(max_width, graph_div) {
        if (this.limitSize_) {
            const window_sz = Util.getViewportSize();
            const window_h = (window_sz[1] - 80);
            const window_w = this.div_contain.offsetWidth ||
                this.div_contain.getBoundingClientRect().width;
            const cvs_sz = this.simCanvas.getScreenRect();
            const cvs_w = cvs_sz.getWidth();
            const cvs_h = cvs_sz.getHeight();
            const limit_w = (window_h / window_w) * (cvs_w / cvs_h);
            max_width = Math.min(max_width, limit_w);
        }
        const widthPct = (Math.floor(max_width * 100) + '%');
        this.div_sim.style.width = widthPct;
        this.div_sim.style.height = 'auto';
        graph_div.style.width = widthPct;
        graph_div.style.height = 'auto';
    }
    ;
    setGraphWidth(value) {
        if (Util.veryDifferent(value, this.graphWidth_)) {
            this.graphWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout.en.GRAPH_WIDTH);
    }
    ;
    setLayout(layout) {
        layout = layout.toLowerCase().trim();
        if (this.layout_ != layout) {
            this.layout_ = layout;
            let tabName = this.layout_;
            switch (tabName) {
                case "sim+graph":
                    tabName = "graph";
                    break;
                case "sim+time_graph":
                    tabName = "time_graph";
                    break;
                case "sim+multi_graph":
                    tabName = "multi_graph";
                    break;
                default:
            }
            this.setSelectedTab(tabName);
            this.redoLayout();
        }
    }
    ;
    setLayoutFromTab(layout) {
        layout = layout.toLowerCase().trim();
        if (this.show_sim_cb.checked) {
            switch (layout) {
                case "graph":
                    layout = "sim+graph";
                    break;
                case "time_graph":
                    layout = "sim+time_graph";
                    break;
                case "multi_graph":
                    layout = "sim+multi_graph";
                    break;
                default:
            }
        }
        this.setLayout(layout);
    }
    ;
    setSelectedTab(layout) {
        if (this.getSelectedTab() != layout) {
            this.tab_list.childNodes.forEach(node => {
                if (node.nodeType != Node.ELEMENT_NODE) {
                    return;
                }
                const elem = node;
                if (elem.tagName != 'LI') {
                    return;
                }
                if (elem.className.trim() == layout) {
                    elem.className += ' selected';
                }
                else {
                    if (elem.className.indexOf('selected') > -1) {
                        elem.className = elem.className.replace(/[ ]*selected/, '');
                    }
                }
            });
        }
    }
    ;
    setSimWidth(value) {
        if (Util.veryDifferent(value, this.simWidth_)) {
            this.simWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout.en.SIM_WIDTH);
    }
    ;
    setTimeGraphWidth(value) {
        if (Util.veryDifferent(value, this.timeGraphWidth_)) {
            this.timeGraphWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout.en.TIME_GRAPH_WIDTH);
    }
    ;
    showSim(visible) {
        switch (this.layout_) {
            case '':
            case "sim":
                break;
            case "graph":
                if (visible) {
                    this.setLayout("sim+graph");
                }
                break;
            case "sim+graph":
                if (!visible) {
                    this.setLayout("graph");
                }
                break;
            case "time_graph":
                if (visible) {
                    this.setLayout("sim+time_graph");
                }
                break;
            case "sim+time_graph":
                if (!visible) {
                    this.setLayout("time_graph");
                }
                break;
            case "multi_graph":
                if (visible) {
                    this.setLayout("sim+multi_graph");
                }
                break;
            case "sim+multi_graph":
                if (!visible) {
                    this.setLayout("multi_graph");
                }
                break;
            default:
                throw 'showSim: no such layout "' + this.layout_ + '"';
        }
    }
    ;
    showTerminal(visible) {
        if (this.terminalEnabled_) {
            this.div_term.style.display = visible ? 'block' : 'none';
            if (this.show_term_cb != null) {
                this.show_term_cb.checked = visible;
            }
            if (visible && this.term_input) {
                this.term_input.focus();
            }
        }
    }
    ;
}
TabLayout.en = {
    SIM_WIDTH: 'sim-width',
    GRAPH_WIDTH: 'graph-width',
    TIME_GRAPH_WIDTH: 'time-graph-width',
    LAYOUT: 'layout',
    SHOW_TERMINAL: 'show terminal'
};
TabLayout.de_strings = {
    SIM_WIDTH: 'Sim Breite',
    GRAPH_WIDTH: 'Graf Breite',
    TIME_GRAPH_WIDTH: 'Zeit Graf Breite',
    LAYOUT: 'layout',
    SHOW_TERMINAL: 'zeige Terminal'
};
TabLayout.i18n = Util.LOCALE === 'de' ? TabLayout.de_strings : TabLayout.en;
;
Util.defineGlobal('sims$common$TabLayout', TabLayout);
