import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { LabCanvas } from '../../lab/view/LabCanvas.js';
import { ParameterBoolean, ParameterNumber, ParameterString } from '../../lab/util/Observe.js';
import { Terminal } from '../../lab/util/Terminal.js';
import { Util } from '../../lab/util/Util.js';
;
export class TabLayout3 extends AbstractSubject {
    constructor(elem_ids, canvasWidth = 800, canvasHeight = 800, opt_terminal = true) {
        super('TAB_LAYOUT');
        this.limitSize_ = true;
        this.simWidth_ = 1;
        this.graphWidth_ = 1;
        this.timeGraphWidth_ = 1;
        this.debug_layout = false;
        this.controls_ = [];
        this.show_term_cb = null;
        Util.setImagesDir(elem_ids.images_dir);
        this.tab_list = Util.getElementById(elem_ids.tab_list);
        this.layout_ = this.getSelectedTab();
        this.terminalEnabled_ = opt_terminal;
        if (this.layout_ == '') {
            this.layout_ = "sim";
            this.setSelectedTab(this.layout_);
        }
        this.tab_list.addEventListener('click', e => {
            const target = e.target;
            if (target === undefined) {
                throw 'event target is undefined';
            }
            if (target == null || target.tagName != 'LI') {
                return;
            }
            if (target.className.indexOf('selected') > -1) {
                return;
            }
            this.setLayoutFromTab(target.className);
        });
        window.addEventListener('resize', () => this.redoLayout());
        const orient = window.screen.orientation;
        if (orient !== undefined) {
            orient.addEventListener('orientationchange', () => this.redoLayout());
        }
        else {
            window.addEventListener('orientationchange', () => this.redoLayout());
        }
        const term_output = Util.maybeElementById(elem_ids.term_output);
        this.term_input = Util.maybeElementById(elem_ids.term_input);
        this.terminal = new Terminal(this.term_input, term_output);
        Terminal.stdRegex(this.terminal);
        this.div_contain = Util.getElementById(elem_ids.container);
        if (this.debug_layout) {
            this.div_contain.style.outline = 'dashed 1px red';
        }
        this.div_sim = Util.getElementById(elem_ids.sim_applet);
        this.div_sim.style.position = 'relative';
        const canvas = document.createElement('canvas');
        canvas.tabIndex = 0;
        this.simCanvas = new LabCanvas(canvas, 'SIM_CANVAS');
        this.simCanvas.setSize(canvasWidth, canvasHeight);
        this.div_sim.insertBefore(this.simCanvas.getCanvas(), this.div_sim.firstChild);
        this.show_sim = Util.getViewportSize()[0] > 600;
        this.show_sim_cb = Util.getElementById(elem_ids.show_sim);
        const p = this.show_sim_cb.parentElement;
        if (p == null || p.tagName != 'LABEL') {
            throw '';
        }
        this.show_sim_cb.checked = this.show_sim;
        this.show_sim_label = p;
        this.show_sim_cb.addEventListener('click', _e => { this.showSim(this.show_sim_cb.checked); });
        this.div_graph = Util.getElementById(elem_ids.div_graph);
        this.div_graph.style.position = 'relative';
        const canvas2 = document.createElement('canvas');
        this.graphCanvas = new LabCanvas(canvas2, 'GRAPH_CANVAS');
        canvasWidth = Math.max(canvasWidth, canvasHeight);
        this.graphCanvas.setSize(canvasWidth, canvasWidth);
        this.div_graph.insertBefore(canvas2, this.div_graph.firstChild);
        this.div_graph_controls =
            Util.getElementById(elem_ids.graph_controls);
        if (this.debug_layout) {
            this.div_graph_controls.style.outline = 'dashed 1px green';
        }
        this.div_sim_controls =
            Util.getElementById(elem_ids.sim_controls);
        if (this.debug_layout) {
            this.div_sim_controls.style.outline = 'dashed 1px green';
        }
        this.div_term =
            Util.getElementById(elem_ids.div_terminal);
        this.div_term.style.display = 'none';
        if (this.debug_layout) {
            this.div_term.style.outline = 'dashed 1px green';
        }
        const label_term = Util.getElementById(elem_ids.label_terminal);
        if (!this.terminalEnabled_) {
            label_term.style.display = 'none';
        }
        else {
            label_term.style.display = 'inline';
            this.show_term_cb =
                Util.getElementById(elem_ids.show_terminal);
            this.show_term_cb.addEventListener('click', _e => {
                if (this.show_term_cb !== null)
                    this.showTerminal(this.show_term_cb.checked);
            });
        }
        this.div_time_graph =
            Util.getElementById(elem_ids.div_time_graph);
        this.div_time_graph.style.position = 'relative';
        const canvas3 = document.createElement('canvas');
        this.timeGraphCanvas = new LabCanvas(canvas3, 'TIME_GRAPH_CANVAS');
        this.timeGraphCanvas.setSize(canvasWidth, canvasWidth);
        this.div_time_graph.insertBefore(canvas3, this.div_time_graph.firstChild);
        this.div_time_graph_controls =
            Util.getElementById(elem_ids.time_graph_controls);
        if (this.debug_layout) {
            this.div_time_graph_controls.style.outline = 'dashed 1px green';
        }
        this.addParameter(new ParameterNumber(this, TabLayout3.en.SIM_WIDTH, TabLayout3.i18n.SIM_WIDTH, () => this.getSimWidth(), a => this.setSimWidth(a)));
        this.addParameter(new ParameterNumber(this, TabLayout3.en.GRAPH_WIDTH, TabLayout3.i18n.GRAPH_WIDTH, () => this.getGraphWidth(), a => this.setGraphWidth(a)));
        this.addParameter(new ParameterNumber(this, TabLayout3.en.TIME_GRAPH_WIDTH, TabLayout3.i18n.TIME_GRAPH_WIDTH, () => this.getTimeGraphWidth(), a => this.setTimeGraphWidth(a)));
        this.addParameter(new ParameterString(this, TabLayout3.en.LAYOUT, TabLayout3.i18n.LAYOUT, () => this.getLayout(), a => this.setLayout(a), TabLayout3.getLayoutValues(), TabLayout3.getLayoutValues()));
        this.addParameter(new ParameterBoolean(this, TabLayout3.en.SHOW_SIM, TabLayout3.i18n.SHOW_SIM, () => this.show_sim, a => this.showSim(a)));
        if (this.terminalEnabled_) {
            this.addParameter(new ParameterBoolean(this, TabLayout3.en.SHOW_TERMINAL, TabLayout3.i18n.SHOW_TERMINAL, () => this.show_term_cb != null ? this.show_term_cb.checked : false, a => this.showTerminal(a)));
        }
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', layout_: "' + this.layout_ + '"'
            + ', simWidth_: ' + Util.NF(this.simWidth_)
            + ', graphWidth_: ' + Util.NF(this.graphWidth_)
            + ', timeGraphWidth_: ' + Util.NF(this.timeGraphWidth_)
            + ', simCanvas: ' + this.simCanvas.toStringShort()
            + ', graphCanvas: ' + this.graphCanvas.toStringShort()
            + ', timeGraphCanvas: ' + this.timeGraphCanvas.toStringShort()
            + ', terminal: ' + this.terminal
            + ', controls_: ['
            + this.controls_.map(a => a.toStringShort())
            + ']'
            + super.toString();
    }
    ;
    getClassName() {
        return 'TabLayout3';
    }
    ;
    static getLayoutValues() {
        return ["sim",
            "graph",
            "sim+graph",
            "time_graph",
            "sim+time_graph",
            "multi_graph",
            "sim+multi_graph"
        ];
    }
    ;
    addControl(control, opt_add = true) {
        if (opt_add) {
            const element = control.getElement();
            this.div_sim_controls.appendChild(element);
        }
        this.controls_.push(control);
        this.redoLayout();
        return control;
    }
    ;
    getGraphCanvas() {
        return this.graphCanvas;
    }
    ;
    getGraphControls() {
        return this.div_graph_controls;
    }
    ;
    getGraphDiv() {
        return this.div_graph;
    }
    ;
    getGraphWidth() {
        return this.graphWidth_;
    }
    ;
    getLayout() {
        return this.layout_;
    }
    ;
    getSelectedTab() {
        let tab = undefined;
        this.tab_list.childNodes.forEach(n => {
            if (n.nodeType == Node.ELEMENT_NODE) {
                const elem = n;
                if (elem.tagName == 'LI') {
                    if (elem.className.match(/.*selected/) != null) {
                        tab = elem;
                    }
                }
            }
        });
        if (tab === undefined) {
            return '';
        }
        const tab2 = tab;
        return tab2.className.replace(/[ ]*selected/, '');
    }
    ;
    getSimCanvas() {
        return this.simCanvas;
    }
    ;
    getSimControls() {
        return this.div_sim_controls;
    }
    ;
    getSimDiv() {
        return this.div_sim;
    }
    ;
    getSimWidth() {
        return this.simWidth_;
    }
    ;
    getSubjects() {
        return [this, this.simCanvas, this.graphCanvas, this.timeGraphCanvas];
    }
    ;
    getTerminal() {
        return this.terminal;
    }
    ;
    getTimeGraphCanvas() {
        return this.timeGraphCanvas;
    }
    ;
    getTimeGraphControls() {
        return this.div_time_graph_controls;
    }
    ;
    getTimeGraphDiv() {
        return this.div_time_graph;
    }
    ;
    getTimeGraphWidth() {
        return this.timeGraphWidth_;
    }
    ;
    setControlsColumns(viewDiv, canvas, controlsDiv) {
        controlsDiv.style.columnCount = '1';
        controlsDiv.style.columnGap = '5px';
        const gap = 5;
        controlsDiv.style.width = 'auto';
        let availWidth;
        if (controlsDiv.offsetLeft > canvas.offsetWidth) {
            availWidth = viewDiv.offsetWidth - canvas.offsetWidth;
        }
        else {
            availWidth = viewDiv.offsetWidth;
        }
        let maxw = 0;
        for (let i = 0; i < controlsDiv.children.length; i++) {
            const e = controlsDiv.children[i];
            let w = e.offsetWidth;
            const sty = window.getComputedStyle(e);
            if (sty) {
                w += parseFloat(sty.marginLeft);
            }
            maxw = Math.max(maxw, w);
        }
        let cols = 1;
        const extra = 4;
        if (availWidth > 2 * maxw + gap + extra) {
            cols = 2;
            if (availWidth > 3 * maxw + 2 * gap + extra) {
                cols = 3;
            }
        }
        controlsDiv.style.columnCount = cols.toString();
    }
    ;
    redoLayout() {
        const view_width = Util.getViewportSize()[0];
        this.div_contain.style.flexDirection = 'row';
        this.div_contain.style.flexWrap = 'nowrap';
        const simBig = (this.simWidth_ * 95).toFixed(1);
        const simSmall = (this.simWidth_ * 47.5).toFixed(1);
        const graphBig = (this.graphWidth_ * 95).toFixed(1);
        const graphSmall = (this.graphWidth_ * 47.5).toFixed(1);
        const timeGraphBig = (this.timeGraphWidth_ * 95).toFixed(1);
        const timeGraphSmall = (this.timeGraphWidth_ * 47.5).toFixed(1);
        let sz;
        switch (this.layout_) {
            case '':
            case "sim":
                this.div_sim.style.display = 'block';
                this.div_sim_controls.style.display = 'inline-block';
                this.div_graph.style.display = 'none';
                this.div_time_graph.style.display = 'none';
                this.show_sim_label.style.display = 'none';
                this.simCanvas.getCanvas().style.maxWidth = simBig + 'vw';
                this.simCanvas.getCanvas().style.maxHeight = simBig + 'vh';
                this.setControlsColumns(this.div_sim, this.simCanvas.getCanvas(), this.div_sim_controls);
                break;
            case "graph":
                this.div_graph.style.display = 'block';
                this.div_graph_controls.style.display = 'inline-block';
                this.div_sim.style.display = 'none';
                this.div_time_graph.style.display = 'none';
                this.graphCanvas.getCanvas().style.maxWidth = graphBig + 'vw';
                this.graphCanvas.getCanvas().style.maxHeight = graphBig + 'vh';
                this.setControlsColumns(this.div_graph, this.graphCanvas.getCanvas(), this.div_graph_controls);
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+graph":
                this.div_sim.style.display = 'block';
                this.div_graph.style.display = 'block';
                this.div_time_graph.style.display = 'none';
                this.div_sim_controls.style.display = 'block';
                this.div_graph_controls.style.display = 'block';
                if (view_width > 600) {
                    this.simCanvas.getCanvas().style.maxWidth = simSmall + 'vw';
                    this.graphCanvas.getCanvas().style.maxWidth = graphSmall + 'vw';
                }
                else {
                    this.div_contain.style.flexDirection = 'column';
                    this.simCanvas.getCanvas().style.maxWidth = simBig + 'vw';
                    this.simCanvas.getCanvas().style.maxHeight = simBig + 'vh';
                    this.graphCanvas.getCanvas().style.maxWidth = graphBig + 'vw';
                    this.graphCanvas.getCanvas().style.maxHeight = graphBig + 'vh';
                }
                this.setControlsColumns(this.div_sim, this.simCanvas.getCanvas(), this.div_sim_controls);
                this.setControlsColumns(this.div_graph, this.graphCanvas.getCanvas(), this.div_graph_controls);
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            case "time_graph":
                this.div_time_graph.style.display = 'block';
                this.div_time_graph_controls.style.display = 'inline-block';
                this.div_sim.style.display = 'none';
                this.div_graph.style.display = 'none';
                this.timeGraphCanvas.getCanvas().style.maxWidth = timeGraphBig + 'vw';
                this.timeGraphCanvas.getCanvas().style.maxHeight = timeGraphBig + 'vh';
                this.setControlsColumns(this.div_time_graph, this.timeGraphCanvas.getCanvas(), this.div_time_graph_controls);
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+time_graph":
                this.div_sim.style.display = 'block';
                this.div_time_graph.style.display = 'block';
                this.div_graph.style.display = 'none';
                this.div_sim_controls.style.display = 'block';
                this.div_time_graph_controls.style.display = 'block';
                if (view_width > 600) {
                    this.simCanvas.getCanvas().style.maxWidth = simSmall + 'vw';
                    this.timeGraphCanvas.getCanvas().style.maxWidth = timeGraphSmall + 'vw';
                }
                else {
                    this.div_contain.style.flexDirection = 'column';
                    this.simCanvas.getCanvas().style.maxWidth = simBig + 'vw';
                    this.simCanvas.getCanvas().style.maxHeight = simBig + 'vh';
                    this.timeGraphCanvas.getCanvas().style.maxWidth = timeGraphBig + 'vw';
                    this.timeGraphCanvas.getCanvas().style.maxHeight = timeGraphBig + 'vh';
                }
                this.setControlsColumns(this.div_sim, this.simCanvas.getCanvas(), this.div_sim_controls);
                this.setControlsColumns(this.div_time_graph, this.timeGraphCanvas.getCanvas(), this.div_time_graph_controls);
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            case "multi_graph":
                this.div_graph.style.display = 'block';
                this.div_time_graph.style.display = 'block';
                this.div_sim.style.display = 'none';
                this.div_sim_controls.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.div_contain.style.flexDirection = view_width > 600 ? 'row' : 'column';
                sz = (view_width > 600) ? '47.5vw' : '95vw';
                this.graphCanvas.getCanvas().style.maxWidth = sz;
                this.timeGraphCanvas.getCanvas().style.maxWidth = sz;
                this.show_sim_cb.checked = false;
                this.show_sim_label.style.display = 'inline';
                break;
            case "sim+multi_graph":
                this.div_graph.style.display = 'block';
                this.div_time_graph.style.display = 'block';
                this.div_sim.style.display = 'block';
                this.div_sim_controls.style.display = 'none';
                this.div_graph_controls.style.display = 'none';
                this.div_time_graph_controls.style.display = 'none';
                this.div_contain.style.flexWrap = 'wrap';
                sz = (view_width > 600) ? '47.5vw' : '95vw';
                this.graphCanvas.getCanvas().style.maxWidth = sz;
                this.timeGraphCanvas.getCanvas().style.maxWidth = sz;
                this.simCanvas.getCanvas().style.maxWidth = sz;
                this.show_sim_cb.checked = true;
                this.show_sim_label.style.display = 'inline';
                break;
            default:
                throw 'redoLayout: no such layout "' + this.layout_ + '"';
        }
    }
    ;
    setGraphWidth(value) {
        if (Util.veryDifferent(value, this.graphWidth_)) {
            this.graphWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout3.en.GRAPH_WIDTH);
    }
    ;
    setLayout(layout) {
        layout = layout.toLowerCase().trim();
        if (this.layout_ != layout) {
            this.layout_ = layout;
            let tabName = this.layout_;
            switch (tabName) {
                case "sim+graph":
                    tabName = "graph";
                    break;
                case "sim+time_graph":
                    tabName = "time_graph";
                    break;
                case "sim+multi_graph":
                    tabName = "multi_graph";
                    break;
                default:
            }
            this.setSelectedTab(tabName);
            this.redoLayout();
        }
    }
    ;
    setLayoutFromTab(layout) {
        layout = layout.toLowerCase().trim();
        if (this.show_sim) {
            switch (layout) {
                case "graph":
                    layout = "sim+graph";
                    break;
                case "time_graph":
                    layout = "sim+time_graph";
                    break;
                case "multi_graph":
                    layout = "sim+multi_graph";
                    break;
                default:
            }
        }
        this.setLayout(layout);
    }
    ;
    setSelectedTab(layout) {
        if (this.getSelectedTab() != layout) {
            this.tab_list.childNodes.forEach(node => {
                if (node.nodeType != Node.ELEMENT_NODE) {
                    return;
                }
                const elem = node;
                if (elem.tagName != 'LI') {
                    return;
                }
                if (elem.className.trim() == layout) {
                    elem.className += ' selected';
                }
                else {
                    if (elem.className.indexOf('selected') > -1) {
                        elem.className = elem.className.replace(/[ ]*selected/, '');
                    }
                }
            });
        }
    }
    ;
    setSimWidth(value) {
        if (Util.veryDifferent(value, this.simWidth_)) {
            this.simWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout3.en.SIM_WIDTH);
    }
    ;
    setTimeGraphWidth(value) {
        if (Util.veryDifferent(value, this.timeGraphWidth_)) {
            this.timeGraphWidth_ = value;
        }
        this.redoLayout();
        this.broadcastParameter(TabLayout3.en.TIME_GRAPH_WIDTH);
    }
    ;
    showSim(visible) {
        this.show_sim = visible;
        switch (this.layout_) {
            case '':
            case "sim":
                break;
            case "graph":
                if (visible) {
                    this.setLayout("sim+graph");
                }
                break;
            case "sim+graph":
                if (!visible) {
                    this.setLayout("graph");
                }
                break;
            case "time_graph":
                if (visible) {
                    this.setLayout("sim+time_graph");
                }
                break;
            case "sim+time_graph":
                if (!visible) {
                    this.setLayout("time_graph");
                }
                break;
            case "multi_graph":
                if (visible) {
                    this.setLayout("sim+multi_graph");
                }
                break;
            case "sim+multi_graph":
                if (!visible) {
                    this.setLayout("multi_graph");
                }
                break;
            default:
                throw 'showSim: no such layout "' + this.layout_ + '"';
        }
    }
    ;
    showTerminal(visible) {
        if (this.terminalEnabled_) {
            this.div_term.style.display = visible ? 'block' : 'none';
            if (this.show_term_cb != null) {
                this.show_term_cb.checked = visible;
            }
            if (visible && this.term_input) {
                this.term_input.focus();
            }
        }
    }
    ;
}
TabLayout3.en = {
    SIM_WIDTH: 'sim-width',
    GRAPH_WIDTH: 'graph-width',
    TIME_GRAPH_WIDTH: 'time-graph-width',
    LAYOUT: 'layout',
    SHOW_TERMINAL: 'show terminal',
    SHOW_SIM: 'show simulation'
};
TabLayout3.de_strings = {
    SIM_WIDTH: 'Sim Breite',
    GRAPH_WIDTH: 'Graf Breite',
    TIME_GRAPH_WIDTH: 'Zeit Graf Breite',
    LAYOUT: 'layout',
    SHOW_TERMINAL: 'zeige Terminal',
    SHOW_SIM: 'zeige Simulation'
};
TabLayout3.i18n = Util.LOCALE === 'de' ? TabLayout3.de_strings : TabLayout3.en;
Util.defineGlobal('sims$common$TabLayout3', TabLayout3);
