import { AutoScale } from '../../lab/graph/AutoScale.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CommonControls } from './CommonControls.js';
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
export class TimeGraph2 {
    constructor(varsList, graphCanvas, div_controls, _div_graph, simRun) {
        this.canvas = graphCanvas;
        simRun.addCanvas(graphCanvas);
        this.view1 = new SimView('TIME_GRAPH_1', new DoubleRect(0, 0, 1, 1));
        this.view1.setHorizAlign("FULL");
        this.view1.setVerticalAlign("FULL");
        graphCanvas.addView(this.view1);
        this.displayList1 = this.view1.getDisplayList();
        this.axes1 = CommonControls.makeAxes(this.view1);
        this.axes1.setColor('lime');
        this.line1 = new GraphLine('TIME_LINE_1', varsList);
        new GenericObserver(this.line1, evt => {
            if (evt.nameEquals(GraphLine.en.X_VARIABLE)) {
                this.axes1.setHorizName(this.line1.getXVarName());
            }
            if (evt.nameEquals(GraphLine.en.Y_VARIABLE)) {
                this.axes1.setVerticalName(this.line1.getYVarName());
            }
        }, 'update axes1 names');
        this.line1.setColor('lime');
        this.view1.addMemo(this.line1);
        this.autoScale1 = new AutoScale('TIME_GRAPH_AUTO_SCALE_1', this.line1, this.view1);
        this.autoScale1.extraMargin = 0.05;
        this.displayGraph1 = new DisplayGraph(this.line1);
        this.displayGraph1.setScreenRect(this.view1.getScreenRect());
        this.displayGraph1.setUseBuffer(false);
        this.displayList1.prepend(this.displayGraph1);
        new GenericObserver(this.view1, evt => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayGraph1.setScreenRect(this.view1.getScreenRect());
            }
        }, 'resize DisplayGraph1');
        const timeIdx = this.line1.getVarsList().timeIndex();
        this.line1.setXVariable(timeIdx);
        this.line1.setYVariable(0);
        this.view2 = new SimView('TIME_GRAPH_2', new DoubleRect(0, 0, 1, 1));
        this.view2.setHorizAlign("FULL");
        this.view2.setVerticalAlign("FULL");
        graphCanvas.addView(this.view2);
        this.displayList2 = this.view2.getDisplayList();
        this.axes2 = CommonControls.makeAxes(this.view2);
        this.axes2.setColor('red');
        this.axes2.setYAxisAlignment("RIGHT");
        this.line2 = new GraphLine('TIME_LINE_2', varsList);
        new GenericObserver(this.line2, evt => {
            if (evt.nameEquals(GraphLine.en.X_VARIABLE)) {
                this.axes2.setHorizName(this.line2.getXVarName());
            }
            if (evt.nameEquals(GraphLine.en.Y_VARIABLE)) {
                this.axes2.setVerticalName(this.line2.getYVarName());
            }
        }, 'update axes2 names');
        this.view2.addMemo(this.line2);
        this.line2.setXVariable(timeIdx);
        this.line2.setYVariable(1);
        this.line2.setColor('red');
        this.autoScale2 = new AutoScale('TIME_GRAPH_AUTO_SCALE_2', this.line2, this.view2);
        this.autoScale2.extraMargin = 0.05;
        this.displayGraph2 = new DisplayGraph(this.line2);
        this.displayGraph2.setScreenRect(this.view2.getScreenRect());
        this.displayGraph2.setUseBuffer(false);
        this.displayList2.prepend(this.displayGraph2);
        new GenericObserver(this.view2, evt => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayGraph2.setScreenRect(this.view2.getScreenRect());
            }
        }, 'resize DisplayGraph2');
        this.controls_ = [];
        this.div_controls = div_controls;
        this.addControl(CommonControls.makePlaybackControls(simRun));
        let pn = this.line1.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, TimeGraph2.i18n.LIME));
        pn = this.line2.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, TimeGraph2.i18n.RED));
        pn = this.line1.getParameterNumber(GraphLine.en.X_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'X:'));
        pn = this.autoScale1.getParameterNumber(AutoScale.en.TIME_WINDOW);
        this.addControl(new NumericControl(pn));
        const bc = new ButtonControl(GraphLine.i18n.CLEAR_GRAPH, () => {
            this.line1.reset();
            this.line2.reset();
        });
        this.addControl(bc);
        this.auto1Obs = new GenericObserver(this.autoScale1, _evt => this.autoScale2.setTimeWindow(this.autoScale1.getTimeWindow()), 'ensures autoScale2 has same time window as autoScale1');
        this.line1Obs = new GenericObserver(this.line1, _evt => {
            const isTimeGraph = this.line1.getXVariable() == timeIdx;
            this.displayGraph1.setUseBuffer(!isTimeGraph);
            this.displayGraph2.setUseBuffer(!isTimeGraph);
            this.line2.setXVariable(this.line1.getXVariable());
        }, 'ensures line2 has same X variable as line1');
    }
    ;
    toString() {
        return 'TimeGraph2{'
            + 'canvas: ' + this.canvas.toStringShort()
            + ', view1: ' + this.view1.toStringShort()
            + ', view2: ' + this.view2.toStringShort()
            + ', line1: ' + this.line1.toStringShort()
            + ', line2: ' + this.line2.toStringShort()
            + ', axes1: ' + this.axes1.toStringShort()
            + ', axes2: ' + this.axes2.toStringShort()
            + ', autoScale1: ' + this.autoScale1.toStringShort()
            + ', autoScale2: ' + this.autoScale2.toStringShort()
            + ', displayGraph1: ' + this.displayGraph1.toStringShort()
            + ', displayGraph2: ' + this.displayGraph2.toStringShort()
            + '}';
    }
    ;
    toStringShort() {
        return 'TimeGraph2{}';
    }
    ;
    addControl(control) {
        const element = control.getElement();
        element.style.display = 'block';
        this.div_controls.appendChild(element);
        this.controls_.push(control);
        return control;
    }
    ;
    getSubjects() {
        return [this.line1, this.line2, this.view1, this.view2, this.autoScale1,
            this.autoScale2];
    }
    ;
}
TimeGraph2.en = {
    LIME: 'lime',
    RED: 'red'
};
TimeGraph2.de_strings = {
    LIME: 'hell grün',
    RED: 'rot'
};
TimeGraph2.i18n = Util.LOCALE === 'de' ? TimeGraph2.de_strings : TimeGraph2.en;
Util.defineGlobal('sims$common$TimeGraph2', TimeGraph2);
