import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { LabCanvas } from '../../lab/view/LabCanvas.js';
import { Terminal } from '../../lab/util/Terminal.js';
import { Util } from '../../lab/util/Util.js';
export class VerticalLayout extends AbstractSubject {
    constructor(elem_ids) {
        super('VERTICAL_LAYOUT');
        this.debug_layout = false;
        this.controls_ = [];
        Util.setImagesDir(elem_ids.images_dir);
        const term_output = Util.maybeElementById(elem_ids.term_output);
        const term_input = Util.maybeElementById(elem_ids.term_input);
        this.terminal = new Terminal(term_input, term_output);
        Terminal.stdRegex(this.terminal);
        this.div_sim = Util.getElementById(elem_ids.sim_applet);
        this.div_sim.style.position = 'relative';
        const canvas = document.createElement('canvas');
        canvas.tabIndex = 0;
        canvas.width = 800;
        canvas.height = 480;
        this.simCanvas = new LabCanvas(canvas, 'simCanvas');
        this.div_sim.appendChild(this.simCanvas.getCanvas());
        this.div_graph = Util.getElementById(elem_ids.div_graph);
        const canvas2 = document.createElement('canvas');
        canvas2.style.float = 'left';
        canvas2.style.margin = '0px 15px 15px 0px';
        this.graphCanvas = new LabCanvas(canvas2, 'graphCanvas');
        this.graphCanvas.setSize(480, 480);
        this.div_graph.style.display = 'none';
        if (this.debug_layout) {
            this.div_graph.style.border = 'dashed 1px blue';
        }
        this.graph_controls =
            Util.getElementById(elem_ids.graph_controls);
        this.div_graph.insertBefore(this.graphCanvas.getCanvas(), this.graph_controls);
        const show_graph_cb = Util.getElementById(elem_ids.show_graph);
        this.showGraph = visible => {
            this.div_graph.style.display = visible ? 'block' : 'none';
            show_graph_cb.checked = visible;
        };
        show_graph_cb.addEventListener('click', _e => this.showGraph(show_graph_cb.checked));
        this.sim_controls = Util.getElementById(elem_ids.sim_controls);
        if (this.debug_layout) {
            this.sim_controls.style.border = 'dashed 1px red';
        }
        const show_controls_cb = Util.getElementById(elem_ids.show_controls);
        this.sim_controls.style.display = 'none';
        this.showControls = (visible) => {
            this.sim_controls.style.display = visible ? 'block' : 'none';
            show_controls_cb.checked = visible;
        };
        show_controls_cb.addEventListener('click', _e => this.showControls(show_controls_cb.checked));
        const form_term = Util.getElementById(elem_ids.form_terminal);
        form_term.style.display = 'none';
        if (this.debug_layout) {
            form_term.style.border = 'dashed 1px green';
        }
        const label_term = Util.getElementById(elem_ids.label_terminal);
        if (this.hideTerminal) {
            label_term.style.display = 'none';
        }
        else {
            const show_term_cb = Util.getElementById(elem_ids.show_terminal);
            this.showTerminal = (visible) => {
                form_term.style.display = visible ? 'block' : 'none';
                show_term_cb.checked = visible;
                if (visible && term_input !== null) {
                    term_input.focus();
                }
            };
            show_term_cb.addEventListener('click', _e => this.showTerminal(show_term_cb.checked));
        }
        const show_hide_form = Util.getElementById(elem_ids.show_hide_form);
        if (this.debug_layout) {
            show_hide_form.style.border = 'dashed 1px green';
        }
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', simCanvas: ' + this.simCanvas.toStringShort()
            + ', graphCanvas: ' + this.graphCanvas.toStringShort()
            + ', terminal: ' + this.terminal
            + ', controls_: ['
            + this.controls_.map(a => a.toStringShort())
            + ']'
            + super.toString();
    }
    ;
    getClassName() {
        return 'VerticalLayout';
    }
    ;
    addControl(control, opt_add = true) {
        if (opt_add) {
            const element = control.getElement();
            element.style.display = 'inline-block';
            this.sim_controls.appendChild(element);
        }
        this.controls_.push(control);
        return control;
    }
    ;
    getGraphCanvas() {
        return this.graphCanvas;
    }
    ;
    getGraphControls() {
        return this.graph_controls;
    }
    ;
    getGraphDiv() {
        return this.div_graph;
    }
    ;
    getSubjects() {
        return [this, this.simCanvas, this.graphCanvas];
    }
    ;
    getSimCanvas() {
        return this.simCanvas;
    }
    ;
    getSimControls() {
        return this.sim_controls;
    }
    ;
    getSimDiv() {
        return this.div_sim;
    }
    ;
    getTerminal() {
        return this.terminal;
    }
    ;
    getTimeGraphCanvas() {
        throw Util.NOT_IMPLEMENTED;
    }
    ;
    getTimeGraphControls() {
        throw Util.NOT_IMPLEMENTED;
    }
    ;
    getTimeGraphDiv() {
        throw Util.NOT_IMPLEMENTED;
    }
    ;
}
Util.defineGlobal('sims$common$VerticalLayout', VerticalLayout);
