import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { CommonControls } from '../common/CommonControls.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { RigidBodySim } from '../../lab/engine2D/RigidBodySim.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
;
export class BilliardsApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        this.offset = 0;
        this.speed = 30;
        this.formation = 1;
        this.layout.getSimCanvas().setBackground('black');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.elasticity.setElasticity(0.95);
        this.sim.setShowForces(false);
        this.dampingLaw = new DampingLaw(0.1, 0.15, this.simList);
        this.addPlaybackControls();
        let pn;
        this.addParameter(pn = new ParameterNumber(this, BilliardsApp.en.FORMATION, BilliardsApp.i18n.FORMATION, () => this.getFormation(), a => this.setFormation(a), [BilliardsApp.i18n.ONE_HITS_THREE,
            BilliardsApp.i18n.ONE_HITS_SIX], [0,
            1]));
        this.addControl(new ChoiceControl(pn));
        this.addParameter(pn = new ParameterNumber(this, BilliardsApp.en.OFFSET, BilliardsApp.i18n.OFFSET, () => this.getOffset(), a => this.setOffset(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, BilliardsApp.en.SPEED, BilliardsApp.i18n.SPEED, () => this.getSpeed(), a => this.setSpeed(a)));
        this.addControl(new NumericControl(pn));
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        let ps = this.sim.getParameterString(RigidBodySim.en.COLLISION_HANDLING);
        this.addControl(new ChoiceControl(ps));
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'BilliardsApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('dampingLaw', myName + '.');
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw);
    }
    ;
    config() {
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        this.advance.reset();
        BilliardsApp.make(this.sim, this.displayList, this.formation, this.offset, this.speed);
        this.sim.setElasticity(elasticity);
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        this.sim.saveInitialState();
        this.clock.setTime(this.sim.getTime());
        this.clock.setRealTime(this.sim.getTime());
        this.easyScript.update();
    }
    ;
    static make(sim, displayList, formation, offset, speed) {
        const r = 0.5;
        const x1 = (2 * r + sim.getDistanceTol() / 2 + offset) * Math.sqrt(3) / 2.0;
        switch (formation) {
            case 1:
                let body = BilliardsApp.makeBall(4, r);
                body.setPosition(new Vector(2 * x1, 0), 0);
                sim.addBody(body);
                displayList.findShape(body).setFillStyle('aqua');
                body = BilliardsApp.makeBall(5, r);
                body.setPosition(new Vector(2 * x1, 2 * r + sim.getDistanceTol() / 4 + offset / 2), 0);
                sim.addBody(body);
                displayList.findShape(body).setFillStyle('fuchsia');
                body = BilliardsApp.makeBall(6, r);
                body.setPosition(new Vector(2 * x1, -2 * r - sim.getDistanceTol() / 4 - offset / 2), 0);
                sim.addBody(body);
                displayList.findShape(body).setFillStyle('orange');
            case 0:
                const body0 = BilliardsApp.makeBall(0, r);
                body0.setPosition(new Vector(-BilliardsApp.WALL_DISTANCE + 1, 0), 0);
                body0.setVelocity(new Vector(speed, 0), 0);
                sim.addBody(body0);
                let ds = displayList.findShape(body0);
                ds.setStrokeStyle('black');
                ds.setFillStyle('white');
                const body1 = BilliardsApp.makeBall(1, r);
                body1.setPosition(new Vector(0, 0), 0);
                sim.addBody(body1);
                displayList.findShape(body1).setFillStyle('red');
                const body2 = BilliardsApp.makeBall(2, r);
                body2.setPosition(new Vector(x1, r + sim.getDistanceTol() / 4 + offset / 2), 0);
                sim.addBody(body2);
                displayList.findShape(body2).setFillStyle('green');
                const body3 = BilliardsApp.makeBall(3, r);
                body3.setPosition(new Vector(x1, -r - sim.getDistanceTol() / 4 - offset / 2), 0);
                sim.addBody(body3);
                displayList.findShape(body3).setFillStyle('blue');
                break;
            default:
                throw '';
        }
        const sz = 2 * BilliardsApp.WALL_DISTANCE;
        Walls.make(sim, sz, sz, 1.0);
    }
    ;
    static makeBall(num, radius) {
        return Shapes.makeBall(radius, BilliardsApp.en.BALL + num, BilliardsApp.i18n.BALL + num);
    }
    ;
    getFormation() {
        return this.formation;
    }
    ;
    setFormation(value) {
        this.formation = value;
        this.config();
        this.broadcastParameter(BilliardsApp.en.FORMATION);
    }
    ;
    getOffset() {
        return this.offset;
    }
    ;
    setOffset(value) {
        this.offset = value;
        this.config();
        this.broadcastParameter(BilliardsApp.en.OFFSET);
    }
    ;
    getSpeed() {
        return this.speed;
    }
    ;
    setSpeed(value) {
        this.speed = value;
        this.config();
        this.broadcastParameter(BilliardsApp.en.SPEED);
    }
    ;
}
BilliardsApp.WALL_DISTANCE = 6;
BilliardsApp.en = {
    FORMATION: 'formation',
    ONE_HITS_THREE: 'one hits three',
    ONE_HITS_SIX: 'one hits six',
    OFFSET: 'offset',
    SPEED: 'speed',
    BALL: 'ball'
};
BilliardsApp.de_strings = {
    FORMATION: 'Formation',
    ONE_HITS_THREE: 'eins schlägt drei',
    ONE_HITS_SIX: 'eins schlägt sechs',
    OFFSET: 'Abstand',
    SPEED: 'Geschwindigkeit',
    BALL: 'Ball'
};
BilliardsApp.i18n = Util.LOCALE === 'de' ? BilliardsApp.de_strings : BilliardsApp.en;
Util.defineGlobal('sims$engine2D$BilliardsApp', BilliardsApp);
