import { ChainConfig } from './ChainConfig.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { GravityLaw } from '../../lab/model/GravityLaw.js';
import { SimView } from '../../lab/view/SimView.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { GenericObserver, ParameterBoolean, ParameterNumber } from '../../lab/util/Observe.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
export class ChainApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-12, -12, 12, 12);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        this.extraBody = true;
        this.walls = true;
        this.debug_ = false;
        this.sim.setShowForces(true);
        this.sim.setExtraAccel("velocity_joints");
        this.dampingLaw = new DampingLaw(0, 0.15, this.simList);
        this.gravityLaw = new GravityLaw(4, this.simList);
        this.elasticity.setElasticity(0.8);
        this.options = {
            wallPivotX: -5,
            wallPivotY: 4,
            fixedLeft: true,
            fixedRight: true,
            blockWidth: 1.0,
            blockHeight: 3.0,
            numLinks: 7
        };
        this.wallWidth = this.simView.getSimRect().getWidth();
        this.addPlaybackControls();
        let pb;
        let pn;
        this.addParameter(pn = new ParameterNumber(this, ChainConfig.en.NUM_LINKS, ChainConfig.i18n.NUM_LINKS, () => this.getNumLinks(), a => this.setNumLinks(a)));
        pn.setDecimalPlaces(0);
        this.addControl(new NumericControl(pn));
        this.addParameter(pb = new ParameterBoolean(this, ChainConfig.en.WALLS, ChainConfig.i18n.WALLS, () => this.getWalls(), a => this.setWalls(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pn = new ParameterNumber(this, ChainConfig.en.WALL_WIDTH, ChainConfig.i18n.WALL_WIDTH, () => this.getWallWidth(), a => this.setWallWidth(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pb = new ParameterBoolean(this, ChainConfig.en.EXTRA_BODY, ChainConfig.i18n.EXTRA_BODY, () => this.getExtraBody(), a => this.setExtraBody(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, ChainConfig.en.FIXED_LEFT, ChainConfig.i18n.FIXED_LEFT, () => this.getFixedLeft(), a => this.setFixedLeft(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, ChainConfig.en.FIXED_RIGHT, ChainConfig.i18n.FIXED_RIGHT, () => this.getFixedRight(), a => this.setFixedRight(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pn = new ParameterNumber(this, ChainConfig.en.FIXED_LEFT_X, ChainConfig.i18n.FIXED_LEFT_X, () => this.getFixedLeftX(), a => this.setFixedLeftX(a)));
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, ChainConfig.en.BLOCK_LENGTH, ChainConfig.i18n.BLOCK_LENGTH, () => this.getBlockLength(), a => this.setBlockLength(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, ChainConfig.en.BLOCK_WIDTH, ChainConfig.i18n.BLOCK_WIDTH, () => this.getBlockWidth(), a => this.setBlockWidth(a)));
        this.addControl(new NumericControl(pn));
        pn = this.gravityLaw.getParameterNumber(GravityLaw.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        this.watchEnergyChange(pn);
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        if (0 == 1) {
            const sr = this.layout.getSimCanvas().getScreenRect();
            this.layout.getGraphCanvas().setScreenRect(sr);
            this.graph.displayGraph.setScreenRect(sr);
            this.graph.autoScale.setEnabled(false);
            const matcher = new GenericObserver(this.simView, evt => {
                if (evt.nameEquals(SimView.SIM_RECT_CHANGED)) {
                    this.graph.view.setSimRect(this.simView.getSimRect());
                }
            }, 'ensure graph\'s simRect matches simView');
            this.graph.view.setSimRect(this.simView.getSimRect());
        }
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup(this.sim.getBody('chain-2'));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + ', gravityLaw: ' + this.gravityLaw.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'ChainApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('gravityLaw|dampingLaw', myName + '.');
        this.terminal.addRegex('ChainConfig|Engine2DApp', 'sims$engine2D$', false);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw, this.gravityLaw);
    }
    ;
    config() {
        if (this.debug_) {
            this.rbo.protoPolygon.setNameFont('10pt sans-serif');
        }
        this.rbo.protoPolygon.setFillStyle('rgba(255,255,255,0.5)');
        this.rbo.protoPolygon.setStrokeStyle('black');
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        this.advance.reset();
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        this.sim.addForceLaw(this.gravityLaw);
        this.gravityLaw.connect(this.sim.getSimList());
        let r = ChainConfig.makeChain(this.sim, this.options);
        if (this.extraBody) {
            const block = Shapes.makeBlock(1, 3, ChainConfig.en.EXTRA_BODY, ChainConfig.i18n.EXTRA_BODY);
            block.setPosition(new Vector(-4, -4));
            this.sim.addBody(block);
            this.displayList.findShape(block).setFillStyle('blue');
            r = r.union(block.getBoundsWorld());
        }
        if (this.walls) {
            r = r.scale(1.1);
            const wr = DoubleRect.makeCentered(Vector.ORIGIN, this.wallWidth, this.wallWidth);
            const zel = Walls.make2(this.sim, wr.union(r));
            this.gravityLaw.setZeroEnergyLevel(zel);
        }
        this.sim.setElasticity(elasticity);
        this.sim.getVarsList().setTime(0);
        this.sim.saveInitialState();
        this.clock.setTime(0);
        this.clock.setRealTime(0);
        this.easyScript.update();
    }
    ;
    getNumLinks() {
        return this.options.numLinks;
    }
    ;
    setNumLinks(value) {
        this.options.numLinks = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.NUM_LINKS);
    }
    ;
    getWalls() {
        return this.walls;
    }
    ;
    setWalls(value) {
        this.walls = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.WALLS);
    }
    ;
    getWallWidth() {
        return this.wallWidth;
    }
    ;
    setWallWidth(value) {
        this.wallWidth = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.WALL_WIDTH);
    }
    ;
    getExtraBody() {
        return this.extraBody;
    }
    ;
    setExtraBody(value) {
        this.extraBody = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.EXTRA_BODY);
    }
    ;
    getFixedLeft() {
        return this.options.fixedLeft;
    }
    ;
    setFixedLeft(value) {
        this.options.fixedLeft = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.FIXED_LEFT);
    }
    ;
    getFixedRight() {
        return this.options.fixedRight;
    }
    ;
    setFixedRight(value) {
        this.options.fixedRight = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.FIXED_RIGHT);
    }
    ;
    getFixedLeftX() {
        return this.options.wallPivotX;
    }
    ;
    setFixedLeftX(value) {
        this.options.wallPivotX = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.FIXED_LEFT_X);
    }
    ;
    getFixedLeftY() {
        return this.options.wallPivotY;
    }
    ;
    setFixedLeftY(value) {
        this.options.wallPivotY = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.FIXED_LEFT_Y);
    }
    ;
    getBlockLength() {
        return this.options.blockHeight;
    }
    ;
    setBlockLength(value) {
        this.options.blockHeight = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.BLOCK_LENGTH);
    }
    ;
    getBlockWidth() {
        return this.options.blockWidth;
    }
    ;
    setBlockWidth(value) {
        this.options.blockWidth = value;
        this.config();
        this.broadcastParameter(ChainConfig.en.BLOCK_WIDTH);
    }
    ;
}
Util.defineGlobal('sims$engine2D$ChainApp', ChainApp);
