import { ConcreteVertex } from '../../lab/engine2D/ConcreteVertex.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class GearsConfig {
    constructor() {
        throw '';
    }
    ;
    static makeGear(radius, startEdges, opt_name, opt_localName) {
        const p = new Polygon(opt_name, opt_localName);
        GearsConfig.addGear(p, radius, 0.3, 36, 30, 30, startEdges);
        p.finish();
        p.setMomentAboutCM(radius * radius / 2);
        return p;
    }
    ;
    static addGear(p, r1, depth, numTeeth, outPercent, inPercent, startEdges) {
        const wholeTooth = 2 * Math.PI / numTeeth;
        const upSlope = wholeTooth * (100.0 - outPercent - inPercent) / 200.0;
        const downSlope = upSlope;
        const outEdge = wholeTooth * outPercent / 100.0;
        const inEdge = wholeTooth * inPercent / 100.0;
        const in_0 = new Vector(r1, 0);
        const out_0 = new Vector(r1 + depth, 0);
        let in1 = in_0;
        p.startPath(new ConcreteVertex(new Vector(in1.getX(), in1.getY())));
        for (let i = 0; i < numTeeth; i++) {
            const toothAngle = i * wholeTooth;
            const out1 = out_0.rotate(toothAngle + upSlope);
            let outsideIsUp;
            if (Math.abs(in1.getX() - out1.getX()) < 1e-3)
                outsideIsUp = out1.getY() > in1.getY();
            else
                outsideIsUp = in1.getX() > out1.getX();
            const e = p.addStraightEdge(out1, outsideIsUp);
            if (i == 0) {
                startEdges.push(e);
            }
            const out2 = out_0.rotate(toothAngle + upSlope + outEdge);
            if (Math.abs(out2.getX() - out1.getX()) < 1e-3)
                outsideIsUp = out2.getY() > out1.getY();
            else
                outsideIsUp = out2.getX() < out1.getX();
            p.addStraightEdge(out2, outsideIsUp);
            const in2 = in_0.rotate(toothAngle + upSlope + outEdge + downSlope);
            if (Math.abs(out2.getX() - in2.getX()) < 1e-3)
                outsideIsUp = out2.getY() > in2.getY() ? false : true;
            else
                outsideIsUp = in2.getX() < out2.getX();
            p.addStraightEdge(in2, outsideIsUp);
            in1 = in_0.rotate(toothAngle + wholeTooth);
            if (Math.abs(in1.getX() - in2.getX()) < 1e-3)
                outsideIsUp = in1.getY() > in2.getY();
            else
                outsideIsUp = in1.getX() < in2.getX();
            p.addStraightEdge(in1, outsideIsUp);
        }
        p.closePath();
    }
    ;
}
GearsConfig.en = {
    LEFT_GEAR: 'left gear',
    RIGHT_GEAR: 'right gear',
    PINNED_GEARS: 'pinned gears',
    TURNING_FORCE: 'turning force',
    TWO_GEARS: 'two gears'
};
GearsConfig.de_strings = {
    LEFT_GEAR: 'linkes Zahnrad',
    RIGHT_GEAR: 'rechtes Zahnrad',
    PINNED_GEARS: 'verstiftet Zahnräder',
    TURNING_FORCE: 'drehende Kraft',
    TWO_GEARS: 'zwei Zahnräder'
};
GearsConfig.i18n = Util.LOCALE === 'de' ? GearsConfig.de_strings : GearsConfig.en;
Util.defineGlobal('sims$engine2D$GearsConfig', GearsConfig);
