import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ClockTask } from '../../lab/util/Clock.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { CommonControls } from '../common/CommonControls.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { GravityLaw } from '../../lab/model/GravityLaw.js';
import { ModifiedEuler } from '../../lab/model/ModifiedEuler.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterBoolean, ParameterNumber } from '../../lab/util/Observe.js';
import { PileConfig } from './PileConfig.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { RandomLCG } from '../../lab/util/Random.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { SixThrusters } from './SixThrusters.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class PileApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-3, -0.2, 3, 5.2);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, 'PILE_APP');
        this.dampingLaw = new DampingLaw(0.05, 0.15);
        this.gravityLaw = new GravityLaw(10);
        this.twoPiles = false;
        this.squareBlocks = false;
        this.connectedBlocks = false;
        this.numBlocks = 7;
        this.endlessLoop = false;
        this.randomSeed = 0;
        this.buildRNG = new RandomLCG(this.randomSeed);
        this.zeroEnergyLevel = 0;
        this.layout.getSimCanvas().setBackground('black');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.rbo.protoPolygon.setNameColor('gray');
        this.rbo.protoPolygon.setNameFont('10pt sans-serif');
        this.elasticity.setElasticity(0.8);
        this.sim.setShowForces(false);
        this.sim.setDistanceTol(0.01);
        this.sim.setCollisionAccuracy(0.6);
        this.diffEqSolver.setDiffEqSolver(ModifiedEuler.en.NAME);
        this.task = new ClockTask(6, () => this.config());
        this.addPlaybackControls();
        let pb;
        let pn;
        pn = new ParameterNumber(this, PileConfig.en.NUM_BLOCKS, PileConfig.i18n.NUM_BLOCKS, () => this.getNumBlocks(), a => this.setNumBlocks(a));
        pn.setDecimalPlaces(0);
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        this.addParameter(pb = new ParameterBoolean(this, PileConfig.en.TWO_PILES, PileConfig.i18n.TWO_PILES, () => this.getTwoPiles(), a => this.setTwoPiles(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, PileConfig.en.CONNECTED_BLOCKS, PileConfig.i18n.CONNECTED_BLOCKS, () => this.getConnectedBlocks(), a => this.setConnectedBlocks(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, PileConfig.en.SQUARE_BLOCKS, PileConfig.i18n.SQUARE_BLOCKS, () => this.getSquareBlocks(), a => this.setSquareBlocks(a)));
        this.addControl(new CheckBoxControl(pb));
        this.addParameter(pb = new ParameterBoolean(this, PileConfig.en.ENDLESS_LOOP, PileConfig.i18n.ENDLESS_LOOP, () => this.getEndlessLoop(), a => this.setEndlessLoop(a)));
        this.addControl(new CheckBoxControl(pb));
        pn = new ParameterNumber(this, PileConfig.en.LOOP_TIME, PileConfig.i18n.LOOP_TIME, () => this.getLoopTime(), a => this.setLoopTime(a));
        pn.setDecimalPlaces(1);
        this.addParameter(pn);
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, PileConfig.en.RANDOM_SEED, PileConfig.i18n.RANDOM_SEED, () => this.getRandomSeed(), a => this.setRandomSeed(a)));
        pn.setDecimalPlaces(0);
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        pn = this.gravityLaw.getParameterNumber(GravityLaw.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        this.watchEnergyChange(pn);
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        let c = new ButtonControl(PileConfig.i18n.REBUILD, () => this.config());
        this.addControl(c);
        c = new ButtonControl(PileConfig.i18n.ADD_BLOCK, () => this.addBlock());
        this.addControl(c);
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + ', gravityLaw: ' + this.gravityLaw.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'PileApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('gravityLaw|dampingLaw', myName + '.');
        this.terminal.addRegex('PileConfig', 'sims$engine2D$', false);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw, this.gravityLaw);
    }
    ;
    config() {
        this.randomSeed = this.buildRNG.getSeed();
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        Polygon.ID = 1;
        this.advance.reset();
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        this.sim.addForceLaw(this.gravityLaw);
        this.gravityLaw.connect(this.sim.getSimList());
        if (this.endlessLoop) {
            this.clock.addTask(this.task);
        }
        else {
            this.clock.removeTask(this.task);
        }
        const blocks = [];
        if (this.twoPiles) {
            this.zeroEnergyLevel = PileConfig.makeDoubleVPit(this.sim, 5);
            const b2 = PileConfig.makeRandomBlocks(this.sim, this.numBlocks, -7, 10, this.buildRNG, this.squareBlocks);
            b2.forEach((b) => blocks.push(b));
        }
        else {
            this.zeroEnergyLevel = PileConfig.makeVPit(this.sim, 9.348706704297266);
            const half = Math.floor(this.numBlocks / 2);
            const rest = this.numBlocks - half;
            let b2 = PileConfig.makeRandomBlocks(this.sim, rest, -9.9, 19, this.buildRNG, this.squareBlocks);
            b2.forEach((b) => blocks.push(b));
            b2 = PileConfig.makeRandomBlocks(this.sim, half, -9, 16, this.buildRNG, this.squareBlocks);
            b2.forEach((b) => blocks.push(b));
        }
        this.gravityLaw.setZeroEnergyLevel(this.zeroEnergyLevel);
        if (this.connectedBlocks) {
            const connect = PileConfig.makeConnectedBlocks(this.sim, 3, 21, 0);
            const thrustForce1 = SixThrusters.make(10.0, connect[0]);
            this.rbeh.setThrusters(thrustForce1, 'right');
            this.sim.addForceLaw(thrustForce1);
            connect.forEach((b) => blocks.push(b));
        }
        blocks.forEach(b => this.displayList.findShape(b).setFillStyle(PileConfig.getRandomColor()));
        this.sim.setElasticity(elasticity);
        this.sim.modifyObjects();
        this.sim.getVarsList().setTime(0);
        this.sim.saveInitialState();
        this.clock.setTime(0);
        this.clock.setRealTime(0);
        this.easyScript.update();
    }
    ;
    addBlock() {
        const b = this.squareBlocks ? Shapes.makeBlock(1, 1) :
            Shapes.makeRandomPolygon(4, 0.7);
        b.setPosition(new Vector(0, 10));
        this.sim.addBody(b);
        this.displayList.findShape(b).setFillStyle(PileConfig.getRandomColor());
        this.sim.saveInitialState();
    }
    ;
    getLoopTime() {
        return this.task.getTime();
    }
    ;
    setLoopTime(value) {
        this.clock.removeTask(this.task);
        this.task = new ClockTask(value, () => this.config());
        this.clock.addTask(this.task);
        if (this.clock.getTime() > value) {
            this.config();
        }
        this.broadcastParameter(PileConfig.en.LOOP_TIME);
    }
    ;
    getNumBlocks() {
        return this.numBlocks;
    }
    ;
    setNumBlocks(value) {
        this.numBlocks = value;
        this.config();
        this.broadcastParameter(PileConfig.en.NUM_BLOCKS);
    }
    ;
    getTwoPiles() {
        return this.twoPiles;
    }
    ;
    setTwoPiles(value) {
        this.twoPiles = value;
        this.config();
        this.broadcastParameter(PileConfig.en.TWO_PILES);
    }
    ;
    getConnectedBlocks() {
        return this.connectedBlocks;
    }
    ;
    setConnectedBlocks(value) {
        this.connectedBlocks = value;
        this.config();
        this.broadcastParameter(PileConfig.en.CONNECTED_BLOCKS);
    }
    ;
    getEndlessLoop() {
        return this.endlessLoop;
    }
    ;
    setEndlessLoop(value) {
        this.endlessLoop = value;
        this.config();
        this.broadcastParameter(PileConfig.en.ENDLESS_LOOP);
    }
    ;
    getRandomSeed() {
        return this.randomSeed;
    }
    ;
    setRandomSeed(value) {
        this.randomSeed = value;
        this.buildRNG.setSeed(value);
        this.config();
        this.broadcastParameter(PileConfig.en.RANDOM_SEED);
    }
    ;
    getSquareBlocks() {
        return this.squareBlocks;
    }
    ;
    setSquareBlocks(value) {
        this.squareBlocks = value;
        this.config();
        this.broadcastParameter(PileConfig.en.SQUARE_BLOCKS);
    }
    ;
}
Util.defineGlobal('sims$engine2D$PileApp', PileApp);
