import { ConcreteLine } from '../../lab/model/ConcreteLine.js';
import { DisplayConnector } from '../../lab/view/DisplayConnector.js';
import { DisplayLine } from '../../lab/view/DisplayLine.js';
import { DisplayRope } from '../../lab/view/DisplayRope.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { Force } from '../../lab/model/Force.js';
import { Impulse } from '../../lab/model/Impulse.js';
import { Joint } from '../../lab/engine2D/Joint.js';
import { PathEndPoint } from '../../lab/engine2D/PathEndPoint.js';
import { PathJoint } from '../../lab/engine2D/PathJoint.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { RigidBodyEventHandler } from '../../lab/app/RigidBodyEventHandler.js';
import { Rope } from '../../lab/engine2D/Rope.js';
import { SimList } from '../../lab/model/SimList.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
export class RigidBodyObserver {
    constructor(simList, displayList) {
        this.memPairs_ = [];
        this.displayList_ = displayList;
        this.simList_ = simList;
        this.addBodies(simList.toArray());
        simList.addObserver(this);
        let dl = new DisplayLine();
        dl.setThickness(1);
        dl.setColor('blue');
        dl.setZIndex(10);
        this.protoForce = dl;
        dl = new DisplayLine();
        dl.setThickness(1);
        dl.setColor('red');
        dl.setZIndex(10);
        this.protoContactForce = dl;
        dl = new DisplayLine();
        dl.setThickness(2);
        dl.setColor('magenta');
        this.protoThrustForce = dl;
        dl = new DisplayLine();
        dl.setThickness(8);
        this.protoConcreteLine = dl;
        let ds = new DisplaySpring();
        ds.setThickness(3);
        ds.setWidth(0.75);
        this.protoSpring = ds;
        ds = new DisplaySpring();
        ds.setThickness(2);
        ds.setWidth(0.5);
        this.protoDragSpring = ds;
        this.protoRope = new DisplayRope();
        this.protoPolygon = new DisplayShape();
        let dsh = new DisplayShape();
        dsh.setFillStyle('');
        dsh.setStrokeStyle('red');
        dsh.setZIndex(10);
        this.protoCollision = dsh;
        this.protoPointMass = new DisplayShape();
        dsh = new DisplayShape();
        dsh.setFillStyle('lightGray');
        dsh.setZIndex(-1);
        this.protoFixedPolygon = dsh;
        let dc = new DisplayConnector();
        dc.setRadius(2);
        dc.setColor('blue');
        dc.setZIndex(10);
        this.protoJoint = dc;
        dc = new DisplayConnector();
        dc.setRadius(3);
        dc.setColor('red');
        dc.setZIndex(10);
        this.protoPathEndPoint = dc;
    }
    ;
    toString() {
        return 'RigidBodyObserver{'
            + 'ordering: "' + RigidBodyObserver.ordering + '"'
            + ', simList_: ' + this.simList_.toStringShort()
            + ', displayList_: ' + this.displayList_.toStringShort()
            + '}';
    }
    ;
    toStringShort() {
        return 'RigidBodyObserver{}';
    }
    ;
    add_(dispObj, simObj) {
        if (RigidBodyObserver.ordering == 'over') {
            this.displayList_.add(dispObj);
        }
        else {
            this.displayList_.prepend(dispObj);
        }
        this.memPairs_.push({ simObj: simObj, dispObj: dispObj });
    }
    ;
    addBodies(bodies) {
        bodies.forEach(obj => this.addBody(obj));
    }
    ;
    addBody(obj) {
        if (this.displayList_.find(obj)) {
            return;
        }
        if (obj instanceof Polygon) {
            if (isFinite(obj.getMass())) {
                this.add_(new DisplayShape(obj, this.protoPolygon), obj);
            }
            else {
                this.add_(new DisplayShape(obj, this.protoFixedPolygon), obj);
            }
        }
        else if (obj instanceof Impulse) {
            const impulse = obj;
            if (impulse.getName().match(/^IMPULSE2/) != null) {
                return;
            }
            if (impulse.getName().match(/^IMPULSE1/) != null) {
                const radius = Math.sqrt(Math.abs(5 * impulse.getMagnitude()) / Math.PI);
                const width = Math.max(0.02, Math.abs(2 * radius));
                const m = PointMass.makeCircle(width, impulse.getName());
                m.setPosition(impulse.getStartPoint());
                this.add_(new DisplayShape(m, this.protoCollision), obj);
            }
        }
        else if (obj instanceof Force) {
            const f = obj;
            if (f.getName().match(/^CONTACT_FORCE2/) != null) {
                return;
            }
            if (f.getName().match(/^CONTACT_FORCE1/) != null) {
                this.add_(new DisplayLine(f, this.protoContactForce), obj);
            }
            else if (f.getName().match(/^THRUSTER/) != null) {
                this.add_(new DisplayLine(f, this.protoThrustForce), obj);
            }
            else {
                this.add_(new DisplayLine(f, this.protoForce), obj);
            }
        }
        else if (obj instanceof PathEndPoint) {
            this.add_(new DisplayConnector(obj, this.protoPathEndPoint), obj);
        }
        else if (obj instanceof Joint || obj instanceof PathJoint) {
            this.add_(new DisplayConnector(obj, this.protoJoint), obj);
        }
        else if (obj instanceof PointMass) {
            this.add_(new DisplayShape(obj, this.protoPointMass), obj);
        }
        else if (obj instanceof Spring) {
            if (obj.nameEquals(RigidBodyEventHandler.en.DRAG)) {
                this.add_(new DisplaySpring(obj, this.protoDragSpring), obj);
            }
            else {
                this.add_(new DisplaySpring(obj, this.protoSpring), obj);
            }
        }
        else if (obj instanceof Rope) {
            this.add_(new DisplayRope(obj, this.protoRope), obj);
        }
        else if (obj instanceof ConcreteLine) {
            this.add_(new DisplayLine(obj, this.protoConcreteLine), obj);
        }
    }
    ;
    removeBody(obj) {
        const pair = this.memPairs_.find(element => element.simObj === obj);
        if (pair) {
            this.displayList_.remove(pair.dispObj);
            Util.remove(this.memPairs_, pair);
        }
    }
    ;
    observe(event) {
        if (event.getSubject() === this.simList_) {
            const obj = event.getValue();
            if (event.nameEquals(SimList.OBJECT_ADDED)) {
                this.addBody(obj);
            }
            else if (event.nameEquals(SimList.OBJECT_REMOVED)) {
                this.removeBody(obj);
            }
        }
    }
    ;
}
RigidBodyObserver.ordering = 'over';
Util.defineGlobal('sims$engine2D$RigidBodyObserver', RigidBodyObserver);
