import { Clock } from "../../lab/util/Clock.js";
import { DisplayShape } from "../../lab/view/DisplayShape.js";
import { DisplaySpring } from "../../lab/view/DisplaySpring.js";
import { DoubleRect } from "../../lab/util/DoubleRect.js";
import { LabCanvas } from "../../lab/view/LabCanvas.js";
import { PointMass } from "../../lab/model/PointMass.js";
import { Spring } from "../../lab/model/Spring.js";
import { SimView } from "../../lab/view/SimView.js";
import { Timer } from "../../lab/util/Timer.js";
import { Util } from "../../lab/util/Util.js";
import { Vector } from "../../lab/util/Vector.js";
export function makeSimpleApp() {
    Util.setErrorHandler();
    const canvas_div = window.document.getElementById('canvas_div');
    if (!canvas_div) {
        throw '';
    }
    const canvas = document.createElement('canvas');
    const simCanvas = new LabCanvas(canvas, 'canvas1');
    simCanvas.setSize(500, 300);
    canvas_div.appendChild(canvas);
    const block1 = PointMass.makeRectangle(1, 1.5, 'block1');
    const fixedPt = PointMass.makeSquare(1, 'fixed');
    fixedPt.setMass(Infinity);
    fixedPt.setPosition(new Vector(-4, 0));
    const spring1 = new Spring('spring', fixedPt, Vector.ORIGIN, block1, Vector.ORIGIN, 1, 3);
    const simRect = new DoubleRect(-5, -5, 5, 5);
    const simView = new SimView('simView', simRect);
    const shape1 = new DisplayShape(block1);
    shape1.setFillStyle('orange');
    simView.getDisplayList().add(shape1);
    const dspring = new DisplaySpring(spring1);
    dspring.setWidth(1.0);
    dspring.setColorCompressed('yellow');
    dspring.setColorExpanded('blue');
    simView.getDisplayList().add(dspring);
    simCanvas.addView(simView);
    const clock = new Clock();
    const timer = new Timer();
    timer.setCallBack(() => {
        block1.setPosition(new Vector(1 + 3 * Math.sin(3 * clock.getTime()), 0));
        simCanvas.paint();
    });
    timer.startFiring();
    clock.resume();
}
;
Util.defineGlobal('sims$experimental$makeSimpleApp', makeSimpleApp);
