import { Util } from '../../lab/util/Util.js';
export class DisplayMagnetWheel {
    constructor(wheel) {
        this.changed_ = true;
        this.wheel_ = wheel;
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', zIndex: ' + this.getZIndex()
            + '}';
    }
    ;
    toStringShort() {
        return 'DisplayMagnetWheel{wheel_: ' +
            this.wheel_.toStringShort() + '}';
    }
    ;
    contains(p_world) {
        const p_body = this.wheel_.worldToBody(p_world);
        return this.wheel_.getBoundsBody().contains(p_body);
    }
    ;
    draw(context, map) {
        const r = this.wheel_.getRadius();
        context.save();
        const sim_to_screen = map.getAffineTransform();
        const sim_to_screen_units = 1 / map.getScaleX();
        const body_to_screen = sim_to_screen.concatenate(this.wheel_.bodyToWorldTransform());
        body_to_screen.setTransform(context);
        context.beginPath();
        if (typeof context.ellipse === 'function') {
            context.moveTo(r, 0);
            context.ellipse(0, 0, r, r, 0, 0, 2 * Math.PI, false);
        }
        else {
            context.arc(0, 0, r, 0, 2 * Math.PI, false);
            context.closePath();
        }
        context.fillStyle = 'lightGray';
        context.fill();
        context.lineWidth = map.screenToSimScaleX(1);
        context.strokeStyle = 'red';
        context.stroke();
        const mr = 0.1 * r;
        const magnets = this.wheel_.getMagnets();
        for (let i = 0, n = magnets.length; i < n; i++) {
            const c = magnets[i];
            const x = c.getX();
            const y = c.getY();
            context.beginPath();
            if (typeof context.ellipse === 'function') {
                context.ellipse(x, y, mr, mr, 0, 0, 2 * Math.PI, false);
            }
            else {
                context.arc(x, y, mr, 0, 2 * Math.PI, false);
                context.closePath();
            }
            context.fillStyle = 'darkGray';
            context.fill();
        }
        sim_to_screen.setTransform(context);
        const fm = this.wheel_.getFixedMagnet();
        const fmr = 0.1 * r;
        context.beginPath();
        if (typeof context.ellipse === 'function') {
            context.moveTo(fmr + fm.getX(), fm.getY());
            context.ellipse(fm.getX(), fm.getY(), fmr, fmr, 0, 0, 2 * Math.PI, false);
        }
        else {
            context.arc(fm.getX(), fm.getY(), fmr, 0, 2 * Math.PI, false);
            context.closePath();
        }
        context.lineWidth = map.screenToSimScaleX(1);
        context.strokeStyle = 'black';
        context.stroke();
        context.restore();
    }
    ;
    getChanged() {
        if (this.wheel_.getChanged() || this.changed_) {
            this.changed_ = false;
            return true;
        }
        return false;
    }
    ;
    getMassObjects() {
        return [this.wheel_];
    }
    ;
    getPosition() {
        return this.wheel_.getPosition();
    }
    ;
    getSimObjects() {
        return [this.wheel_];
    }
    ;
    getZIndex() {
        if (this.zIndex_ !== undefined) {
            return this.zIndex_;
        }
        else {
            return 0;
        }
    }
    ;
    isDragable() {
        return false;
    }
    ;
    setDragable(_dragable) {
    }
    ;
    setPosition(_position) {
    }
    ;
    setZIndex(zIndex) {
        this.zIndex_ = zIndex;
        this.changed_ = true;
    }
    ;
}
Util.defineGlobal('sims$misc$DisplayMagnetWheel', DisplayMagnetWheel);
