import { AbstractMassObject } from '../../lab/model/MassObject.js';
import { adaptQuad } from '../../lab/util/Calculus.js';
import { Force } from '../../lab/model/Force.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class MagnetWheel extends AbstractMassObject {
    constructor(radius = 1, opt_name, opt_localName) {
        let name, localName;
        if (opt_name === undefined || opt_name == '') {
            const id = MagnetWheel.wheel_ID++;
            name = MagnetWheel.en.MAGNET_WHEEL + id;
            localName = MagnetWheel.i18n.MAGNET_WHEEL + id;
        }
        else {
            name = opt_name;
            localName = opt_localName ?? name;
        }
        super(name, localName);
        this.fixedMagnet_ = new Vector(0, 0.9);
        this.magnetStrength_ = 1;
        this.magnets_ = [];
        this.magnetDist_ = [];
        this.mass_ = 1;
        this.moment_ = 1;
        this.radius_ = radius;
    }
    ;
    toString() {
        return super.toString().slice(0, -1)
            + ', fixedMagnet_: ' + this.fixedMagnet_
            + ', magnetStrength_: ' + Util.NF(this.magnetStrength_)
            + ', num magnets: ' + this.magnets_.length
            + ', radius_: ' + Util.NF(this.radius_)
            + '}';
    }
    ;
    getClassName() {
        return 'MagnetWheel';
    }
    ;
    calculateForces() {
        const fm = this.fixedMagnet_;
        const forces = [];
        for (let i = 0, n = this.magnets_.length; i < n; i++) {
            const r = this.bodyToWorld(this.magnets_[i]);
            let f = new Vector(fm.getX() - r.getX(), fm.getY() - r.getY());
            f = f.normalize().multiply(this.magnetStrength_ / f.lengthSquared());
            const t = r.getX() * f.getY() - r.getY() * f.getX();
            const fc = new Force('magnet' + i, this, r, 1, f, 1, t);
            forces.push(fc);
        }
        return forces;
    }
    ;
    createCanvasPath(_context) {
    }
    ;
    disconnect() {
    }
    ;
    getBodies() {
        return [this];
    }
    ;
    getBottomBody() {
        return -this.radius_;
    }
    ;
    getCentroidBody() {
        return Vector.ORIGIN;
    }
    ;
    getCentroidRadius() {
        return this.radius_;
    }
    ;
    getLeftBody() {
        return -this.radius_;
    }
    ;
    getFixedMagnet() {
        return this.fixedMagnet_;
    }
    ;
    getMagnets() {
        return Array.from(this.magnets_);
    }
    ;
    getMagnetStrength() {
        return this.magnetStrength_;
    }
    ;
    getMinHeight() {
        return this.radius_;
    }
    ;
    getPotentialEnergy() {
        let pe = 0;
        for (let i = 0, n = this.magnets_.length; i < n; i++) {
            const r = this.bodyToWorld(this.magnets_[i]);
            let a = r.getAngle();
            if (a < -Math.PI / 2) {
                a = a + 2 * Math.PI;
            }
            if (a < Math.PI / 2) {
                pe += Math.abs(adaptQuad(angle => this.getTorque(i, angle), a, Math.PI / 2, 0.0001));
            }
            else {
                pe += Math.abs(adaptQuad(angle => this.getTorque(i, angle), Math.PI / 2, a, 0.0001));
            }
        }
        ;
        return pe;
    }
    ;
    getRadius() {
        return this.radius_;
    }
    getRightBody() {
        return this.radius_;
    }
    ;
    getTopBody() {
        return this.radius_;
    }
    ;
    getTorque(idx, angle) {
        if (idx < 0 || idx >= this.magnetDist_.length)
            throw '';
        const fm = this.fixedMagnet_;
        const d = this.magnetDist_[idx];
        const r = new Vector(d * Math.cos(angle), d * Math.sin(angle));
        let f = new Vector(fm.getX() - r.getX(), fm.getY() - r.getY());
        f = f.normalize().multiply(this.magnetStrength_ / f.lengthSquared());
        return r.getX() * f.getY() - r.getY() * f.getX();
    }
    ;
    getVerticesBody() {
        const w = this.radius_;
        const h = this.radius_;
        return [new Vector(-w, -h), new Vector(w, -h), new Vector(w, h), new Vector(-w, h)];
    }
    ;
    setFixedMagnet(loc) {
        if (loc.getX() != 0 || loc.getY() <= 0) {
            throw 'fixed magnet must have X = 0, and Y > 0';
        }
        this.fixedMagnet_ = loc;
        this.setChanged();
        return this;
    }
    ;
    setMagnets(locations) {
        this.magnets_ = Array.from(locations);
        this.magnetDist_ = [];
        this.magnets_.forEach(m => this.magnetDist_.push(m.length()));
        this.setChanged();
    }
    ;
    setMagnetStrength(value) {
        this.magnetStrength_ = value;
        this.setChanged();
    }
    ;
    setPosition(loc_world, angle) {
        if (!Vector.ORIGIN.equals(loc_world)) {
            throw 'MagnetWheel can only be located at origin';
        }
        super.setPosition(loc_world, angle);
    }
    ;
    setRadius(radius) {
        this.radius_ = radius;
        this.setChanged();
        return this;
    }
    ;
    similar(_obj, _opt_tolerance) {
        return false;
    }
    ;
}
MagnetWheel.wheel_ID = 1;
MagnetWheel.en = {
    MAGNET_WHEEL: 'MagnetWheel'
};
MagnetWheel.de_strings = {
    MAGNET_WHEEL: 'MagnetRad'
};
MagnetWheel.i18n = Util.LOCALE === 'de' ? MagnetWheel.de_strings : MagnetWheel.en;
Util.defineGlobal('sims$misc$MagnetWheel', MagnetWheel);
