import { BrachistoSim } from './BrachistoSim.js';
import { DisplayPath } from '../../lab/view/DisplayPath.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplayText } from '../../lab/view/DisplayText.js';
import { DrawingStyle } from '../../lab/view/DrawingStyle.js';
import { NumericalPath } from '../../lab/model/NumericalPath.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { SimList } from '../../lab/model/SimList.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class BrachistoObserver {
    constructor(sim, simList, simView, statusView) {
        this.sim_ = sim;
        this.simView_ = simView;
        this.displayList_ = simView.getDisplayList();
        this.statusView_ = statusView;
        this.displayPath_ = new DisplayPath();
        this.displayPath_.setScreenRect(simView.getScreenRect());
        this.displayPath_.setZIndex(-10);
        this.displayList_.add(this.displayPath_);
        this.simList_ = simList;
        this.addBodies(simList.toArray());
        this.message_ = new DisplayText(BrachistoObserver.i18n.QUESTION);
        this.message_.setFillStyle('gray');
        this.message_.setDragable(false);
        this.message_.setPosition(this.getTextPosition());
        this.statusView_.getDisplayList().add(this.message_);
        simView.addObserver(this);
        simList.addObserver(this);
        sim.addObserver(this);
    }
    ;
    toString() {
        return 'BrachistoObserver{'
            + 'sim_: ' + this.sim_.toStringShort()
            + ', simList_: ' + this.simList_.toStringShort()
            + ', simView_: ' + this.simView_.toStringShort()
            + ', displayList_: ' + this.displayList_.toStringShort()
            + ', statusView_: ' + this.statusView_.toStringShort()
            + ', displayPath_: ' + this.displayPath_.toStringShort()
            + '}';
    }
    ;
    toStringShort() {
        return 'BrachistoObserver{}';
    }
    ;
    addBodies(bodies) {
        bodies.forEach(obj => this.addBody(obj));
    }
    ;
    addBody(obj) {
        const dobj = this.displayList_.find(obj);
        if (dobj) {
            return;
        }
        if (obj instanceof NumericalPath) {
            this.displayPath_.addPath(obj, DrawingStyle.lineStyle('gray', 2));
        }
        else if (obj instanceof PointMass) {
            const pm = obj;
            const dp = new DisplayShape(pm);
            dp.setDragable(false);
            if (pm.getName().match(/BALL_SELECTED/)) {
                dp.setFillStyle('red');
                this.displayList_.add(dp);
            }
            else if (pm.getName().match(/BALL/)) {
                dp.setFillStyle('blue');
                this.displayList_.add(dp);
            }
            else if (pm.getName().match(/(START|END)/)) {
                dp.setFillStyle('');
                dp.setStrokeStyle('gray');
                dp.setThickness(1);
                dp.setZIndex(-1);
                this.displayList_.add(dp);
            }
        }
        else {
            throw 'BrachistoObserver unknown object ' + obj;
        }
    }
    ;
    getTextPosition() {
        const sr = this.statusView_.getScreenRect();
        const r = this.statusView_.getCoordMap().screenToSimRect(sr);
        return new Vector(r.getLeft() + 0.15 * r.getWidth(), r.getTop() - 0.1 * r.getHeight());
    }
    ;
    observe(event) {
        if (event.getSubject() == this.sim_) {
            if (event.nameEquals(BrachistoSim.PATH_CHOSEN)) {
                let msg;
                const choice = this.sim_.getPathChoice();
                if (choice >= 0) {
                    msg = BrachistoObserver.i18n.YOU_PICKED;
                    msg += ' ' + this.sim_.getPaths()[choice].getName();
                    msg += ' ' + BrachistoObserver.i18n.PATH + '.';
                }
                else {
                    msg = BrachistoObserver.i18n.QUESTION;
                }
                this.message_.setText(msg);
            }
        }
        else if (event.getSubject() == this.simView_) {
            if (event.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayPath_.setScreenRect(this.simView_.getScreenRect());
            }
        }
        else if (event.getSubject() == this.simList_) {
            const obj = event.getValue();
            if (event.nameEquals(SimList.OBJECT_ADDED)) {
                this.addBody(obj);
            }
            else if (event.nameEquals(SimList.OBJECT_REMOVED)) {
                if (obj instanceof NumericalPath) {
                    this.displayPath_.removePath(obj);
                }
                else if (obj instanceof PointMass) {
                    const dispObj = this.displayList_.find(obj);
                    if (dispObj) {
                        this.displayList_.remove(dispObj);
                    }
                }
            }
        }
    }
    ;
}
BrachistoObserver.en = {
    PATH: 'path',
    QUESTION: 'Which path is fastest to slide down?  Click a path to begin.',
    YOU_PICKED: 'You picked the'
};
BrachistoObserver.de_strings = {
    PATH: 'Pfad gewählt',
    QUESTION: 'Welcher Pfad ist am schnellsten zum abrutschen?  Wählen Sie ein Pfad um zu beginnen.',
    YOU_PICKED: 'Sie haben den'
};
BrachistoObserver.i18n = Util.LOCALE === 'de' ? BrachistoObserver.de_strings : BrachistoObserver.en;
Util.defineGlobal('sims$roller$BrachistoObserver', BrachistoObserver);
