import { AbstractApp } from '../common/AbstractApp.js';
import { CardioidPath } from './CardioidPath.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CirclePath } from './CirclePath.js';
import { CommonControls } from '../common/CommonControls.js';
import { CustomPath } from './CustomPath.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { FlatPath } from './FlatPath.js';
import { HumpPath } from './HumpPath.js';
import { LemniscatePath } from './LemniscatePath.js';
import { LoopTheLoopPath } from './LoopTheLoopPath.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterNumber, ParameterString } from '../../lab/util/Observe.js';
import { OvalPath } from './OvalPath.js';
import { PathObserver } from './PathObserver.js';
import { PathSelector } from './PathSelector.js';
import { RollerSingleSim } from './RollerSingleSim.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SpiralPath } from './SpiralPath.js';
import { TextControl } from '../../lab/controls/TextControl.js';
import { Util } from '../../lab/util/Util.js';
export class RollerSingleApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new RollerSingleSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.layout.getSimCanvas().setBackground('white');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.ball1 = new DisplayShape(this.simList.getPointMass('ball1'));
        this.ball1.setFillStyle('blue');
        this.displayList.add(this.ball1);
        this.customPath_ = new CustomPath();
        this.paths = [
            new HumpPath(),
            new LoopTheLoopPath(),
            new CirclePath(3.0),
            new OvalPath(),
            new LemniscatePath(2.0),
            new CardioidPath(3.0),
            new SpiralPath(),
            new FlatPath(),
            this.customPath_
        ];
        this.pathSelect = new PathSelector(sim, this.paths);
        this.pathObserver = new PathObserver(this.simList, this.simView, a => this.setSimRect(a));
        this.pathSelect.setPathName(HumpPath.en.NAME);
        this.addPlaybackControls();
        let ps = this.pathSelect.getParameterString(PathSelector.en.PATH);
        this.addControl(new ChoiceControl(ps));
        let pn = sim.getParameterNumber(RollerSingleSim.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(RollerSingleSim.en.MASS);
        this.addControl(new NumericControl(pn));
        this.addParameter(ps = new ParameterString(this, RollerSingleApp.en.EQUATION_X, RollerSingleApp.i18n.EQUATION_X, () => this.getXEquation(), a => this.setXEquation(a))
            .setSuggestedLength(30));
        this.addControl(new TextControl(ps));
        this.addParameter(ps = new ParameterString(this, RollerSingleApp.en.EQUATION_Y, RollerSingleApp.i18n.EQUATION_Y, () => this.getYEquation(), a => this.setYEquation(a))
            .setSuggestedLength(30));
        this.addControl(new TextControl(ps));
        this.addParameter(pn = new ParameterNumber(this, RollerSingleApp.en.START_T_VALUE, RollerSingleApp.i18n.START_T_VALUE, () => this.getStartTValue(), a => this.setStartTValue(a))
            .setLowerLimit(Number.NEGATIVE_INFINITY));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, RollerSingleApp.en.FINISH_T_VALUE, RollerSingleApp.i18n.FINISH_T_VALUE, () => this.getFinishTValue(), a => this.setFinishTValue(a))
            .setLowerLimit(Number.NEGATIVE_INFINITY));
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(1);
        this.timeGraph.line1.setYVariable(0);
        this.timeGraph.line2.setYVariable(1);
        this.makeEasyScript([this.simView]);
        this.addURLScriptButton();
        this.pathSelect.addObserver(this);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', ball1: ' + this.ball1.toStringShort()
            + ', pathSelect: ' + this.pathSelect.toStringShort()
            + ', paths: [ ' + this.paths + ' ]'
            + super.toString();
    }
    ;
    getClassName() {
        return 'RollerSingleApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('ball1|paths|pathSelect', myName + '.', true);
    }
    ;
    getFinishTValue() {
        return this.customPath_.getFinishTValue();
    }
    ;
    getStartTValue() {
        return this.customPath_.getStartTValue();
    }
    ;
    setFinishTValue(value) {
        this.customPath_.setFinishTValue(value);
        this.pathSelect.setPathName(this.customPath_.getName());
        this.pathSelect.update();
        this.broadcastParameter(RollerSingleApp.en.FINISH_T_VALUE);
    }
    ;
    setStartTValue(value) {
        this.customPath_.setStartTValue(value);
        this.pathSelect.setPathName(this.customPath_.getName());
        this.pathSelect.update();
        this.broadcastParameter(RollerSingleApp.en.START_T_VALUE);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.pathSelect);
    }
    ;
    getXEquation() {
        return this.customPath_.getXEquation();
    }
    ;
    getYEquation() {
        return this.customPath_.getYEquation();
    }
    ;
    setXEquation(value) {
        const oldValue = this.getXEquation();
        try {
            this.customPath_.setXEquation(value);
            this.pathSelect.setPathName(this.customPath_.getName());
            this.pathSelect.update();
            this.broadcastParameter(RollerSingleApp.en.EQUATION_X);
        }
        catch (ex) {
            this.customPath_.setXEquation(oldValue);
            this.pathSelect.update();
            throw ex;
        }
    }
    ;
    setYEquation(value) {
        const oldValue = this.getYEquation();
        try {
            this.customPath_.setYEquation(value);
            this.pathSelect.setPathName(this.customPath_.getName());
            this.pathSelect.update();
            this.broadcastParameter(RollerSingleApp.en.EQUATION_Y);
        }
        catch (ex) {
            this.customPath_.setYEquation(oldValue);
            this.pathSelect.update();
            throw ex;
        }
    }
    ;
    observe(event) {
        if (event.getSubject() == this.pathSelect) {
            if (this.pathSelect.getPathName() == 'LEMNISCATE') {
                this.easyScript.parse('velocity=5');
            }
            this.easyScript.update();
            this.sim.modifyObjects();
        }
    }
    ;
    setSimRect(simRect) {
        this.simRect = simRect;
        this.simView.setSimRect(simRect);
    }
    ;
}
RollerSingleApp.en = {
    EQUATION_X: 'X-equation',
    EQUATION_Y: 'Y-equation',
    START_T_VALUE: 'start-t',
    FINISH_T_VALUE: 'finish-t'
};
RollerSingleApp.de_strings = {
    EQUATION_X: 'X-Gleichung',
    EQUATION_Y: 'Y-Gleichung',
    START_T_VALUE: 'anfangs-t',
    FINISH_T_VALUE: 'ende-t'
};
RollerSingleApp.i18n = Util.LOCALE === 'de' ? RollerSingleApp.de_strings : RollerSingleApp.en;
Util.defineGlobal('sims$roller$RollerSingleApp', RollerSingleApp);
