import { AbstractApp } from '../common/AbstractApp.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { Double2DSpringSim } from './Double2DSpringSim.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Util } from '../../lab/util/Util.js';
export class Double2DSpringApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new Double2DSpringSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.layout.getSimCanvas().setBackground('black');
        this.protoBob = new DisplayShape();
        this.protoBob.setFillStyle('blue');
        this.protoSpring = new DisplaySpring();
        this.protoSpring.setWidth(0.3);
        this.protoSpring.setColorCompressed('#0c0');
        this.protoSpring.setColorExpanded('#6f6');
        this.topMass = new DisplayShape(this.simList.getPointMass('top'));
        this.topMass.setFillStyle('red');
        this.displayList.add(this.topMass);
        this.spring1 = new DisplaySpring(this.simList.getSpring('spring1'), this.protoSpring);
        this.displayList.add(this.spring1);
        this.spring2 = new DisplaySpring(this.simList.getSpring('spring2'), this.protoSpring);
        this.displayList.add(this.spring2);
        this.bob1 = new DisplayShape(this.simList.getPointMass('bob1'), this.protoBob);
        this.displayList.add(this.bob1);
        this.bob2 = new DisplayShape(this.simList.getPointMass('bob2'), this.protoBob);
        this.displayList.add(this.bob2);
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(Double2DSpringSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 50, false));
        pn = sim.getParameterNumber(Double2DSpringSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 1, false));
        pn = sim.getParameterNumber(Double2DSpringSim.en.MASS1);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(Double2DSpringSim.en.MASS2);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(Double2DSpringSim.en.LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(Double2DSpringSim.en.STIFFNESS);
        this.addControl(new SliderControl(pn, 0, 200, false));
        pn = sim.getParameterNumber(EnergyInfo.en.PE_OFFSET);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        const bc = new ButtonControl(Double2DSpringSim.i18n.REST_STATE, () => sim.restState());
        this.addControl(bc);
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob1: ' + this.bob1.toStringShort()
            + ', bob2: ' + this.bob2.toStringShort()
            + ', spring1: ' + this.spring1.toStringShort()
            + ', spring2: ' + this.spring2.toStringShort()
            + ', topMass: ' + this.topMass.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'Double2DSpringApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('topMass|bob1|bob2|spring1|spring2|protoBob|protoSpring', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$springs$Double2DSpringApp', Double2DSpringApp);
