import { AbstractODESim } from '../../lab/model/ODESim.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { ParameterBoolean } from '../../lab/util/Observe.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { VarsList } from '../../lab/model/VarsList.js';
import { Vector } from '../../lab/util/Vector.js';
const U1 = 0;
const U2 = 1;
const V1 = 2;
const V2 = 3;
const A1 = 4;
const A2 = 5;
const KE = 6;
const PE = 7;
const TE = 8;
const TIME = 9;
export class DoubleSpringSim extends AbstractODESim {
    constructor(thirdSpring, opt_name) {
        super(opt_name);
        this.stiffness_ = 6.0;
        this.damping_ = 0;
        this.potentialOffset_ = 0;
        this.dragBlock_ = -1;
        const var_names = [
            DoubleSpringSim.en.POSITION + '-1',
            DoubleSpringSim.en.POSITION + '-2',
            DoubleSpringSim.en.VELOCITY + '-1',
            DoubleSpringSim.en.VELOCITY + '-2',
            DoubleSpringSim.en.ACCELERATION + '-1',
            DoubleSpringSim.en.ACCELERATION + '-2',
            EnergyInfo.en.KINETIC_ENERGY,
            EnergyInfo.en.POTENTIAL_ENERGY,
            EnergyInfo.en.TOTAL_ENERGY,
            VarsList.en.TIME
        ];
        const i18n_names = [
            DoubleSpringSim.i18n.POSITION + '-1',
            DoubleSpringSim.i18n.POSITION + '-2',
            DoubleSpringSim.i18n.VELOCITY + '-1',
            DoubleSpringSim.i18n.VELOCITY + '-2',
            DoubleSpringSim.i18n.ACCELERATION + '-1',
            DoubleSpringSim.i18n.ACCELERATION + '-2',
            EnergyInfo.i18n.KINETIC_ENERGY,
            EnergyInfo.i18n.POTENTIAL_ENERGY,
            EnergyInfo.i18n.TOTAL_ENERGY,
            VarsList.i18n.TIME
        ];
        this.setVarsList(new VarsList(var_names, i18n_names, this.getName() + '_VARS'));
        this.getVarsList().setComputed(A1, A2, KE, PE, TE);
        this.block1_ = PointMass.makeSquare(1, 'block1');
        this.block2_ = PointMass.makeSquare(1, 'block2');
        this.wall1_ = PointMass.makeRectangle(0.4, 4, 'wall1');
        this.wall1_.setMass(Infinity);
        this.wall1_.setPosition(new Vector(-0.2, 0));
        this.wall2_ = PointMass.makeRectangle(0.4, 4, 'wall2');
        this.wall2_.setMass(Infinity);
        this.wall2_.setPosition(new Vector(9.8, 0));
        const length = 3.0;
        this.thirdSpring_ = thirdSpring;
        this.spring1_ = new Spring('spring1', this.wall1_, new Vector(this.wall1_.getWidth() / 2.0, 0), this.block1_, Vector.ORIGIN, length, this.stiffness_);
        this.spring2_ = new Spring('spring2', this.block1_, Vector.ORIGIN, this.block2_, Vector.ORIGIN, length, this.stiffness_);
        this.spring3_ = new Spring('spring3', this.block2_, Vector.ORIGIN, this.wall2_, new Vector(-this.wall2_.getWidth() / 2.0, 0), length, (this.thirdSpring_ ? this.stiffness_ : 0));
        this.springs_ = [this.spring1_, this.spring2_, this.spring3_];
        this.getSimList().add(this.wall1_, this.wall2_, this.block1_, this.block2_, this.spring1_, this.spring2_, this.spring3_);
        let pn;
        this.addParameter(new ParameterNumber(this, DoubleSpringSim.en.DAMPING, DoubleSpringSim.i18n.DAMPING, () => this.getDamping(), a => this.setDamping(a)));
        this.addParameter(new ParameterNumber(this, DoubleSpringSim.en.LENGTH, DoubleSpringSim.i18n.LENGTH, () => this.getLength(), a => this.setLength(a)));
        this.addParameter(new ParameterNumber(this, DoubleSpringSim.en.MASS1, DoubleSpringSim.i18n.MASS1, () => this.getMass1(), a => this.setMass1(a)));
        this.addParameter(new ParameterNumber(this, DoubleSpringSim.en.MASS2, DoubleSpringSim.i18n.MASS2, () => this.getMass2(), a => this.setMass2(a)));
        this.addParameter(new ParameterNumber(this, DoubleSpringSim.en.STIFFNESS, DoubleSpringSim.i18n.STIFFNESS, () => this.getStiffness(), a => this.setStiffness(a)));
        this.addParameter(new ParameterBoolean(this, DoubleSpringSim.en.THIRD_SPRING, DoubleSpringSim.i18n.THIRD_SPRING, () => this.getThirdSpring(), a => this.setThirdSpring(a)));
        this.addParameter(pn = new ParameterNumber(this, EnergyInfo.en.PE_OFFSET, EnergyInfo.i18n.PE_OFFSET, () => this.getPEOffset(), a => this.setPEOffset(a)));
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        pn.setSignifDigits(5);
        this.restState();
        this.getVarsList().setValue(V2, -2.3);
        this.saveInitialState();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', block1_: ' + this.block1_
            + ', block2_: ' + this.block2_
            + ', spring1_: ' + this.spring1_
            + ', damping_: ' + Util.NF(this.damping_)
            + ', stiffness_: ' + Util.NF(this.stiffness_)
            + ', thirdSpring_: ' + this.thirdSpring_
            + ', potentialOffset_: ' + Util.NF(this.potentialOffset_)
            + super.toString();
    }
    ;
    getClassName() {
        return 'DoubleSpringSim';
    }
    ;
    restState() {
        const vars = this.getVarsList().getValues();
        const k1 = this.spring1_.getStiffness();
        const k2 = this.spring2_.getStiffness();
        const k3 = this.spring3_.getStiffness();
        const R1 = this.spring1_.getRestLength();
        const R2 = this.spring2_.getRestLength();
        const R3 = this.spring3_.getRestLength();
        const w1 = this.wall1_.getBoundsWorld().getRight();
        const w2 = this.wall2_.getBoundsWorld().getLeft();
        let u1 = (k1 * w1 + k1 * R1 + k3 * w2 + (k3 * k1 / k2) * (w1 + R1) - k3 * R2 - k3 * R3);
        u1 = u1 / (k1 + (k3 * k1 / k2) + k3);
        vars[U1] = u1;
        vars[U2] = (k1 / k2) * (u1 - w1 - R1) + u1 + R2;
        vars[V1] = vars[V2] = 0;
        this.getVarsList().setValues(vars);
        this.moveObjects(vars);
        this.potentialOffset_ = 0;
        this.setPEOffset(-this.getEnergyInfo().getPotential());
    }
    ;
    getEnergyInfo() {
        const ke = this.block1_.getKineticEnergy() + this.block2_.getKineticEnergy();
        let pe = this.potentialOffset_;
        this.springs_.forEach(spr => pe += spr.getPotentialEnergy());
        return new EnergyInfo(pe, ke);
    }
    ;
    getPEOffset() {
        return this.potentialOffset_;
    }
    setPEOffset(value) {
        this.potentialOffset_ = value;
        this.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(EnergyInfo.en.PE_OFFSET);
    }
    ;
    modifyObjects() {
        const va = this.getVarsList();
        const vars = va.getValues();
        this.moveObjects(vars);
        const rate = new Array(vars.length);
        this.evaluate(vars, rate, 0);
        vars[A1] = rate[V1];
        vars[A2] = rate[V2];
        const ei = this.getEnergyInfo();
        vars[KE] = ei.getTranslational();
        vars[PE] = ei.getPotential();
        vars[TE] = ei.getTotalEnergy();
        va.setValues(vars, true);
    }
    ;
    moveObjects(vars) {
        this.block1_.setPosition(new Vector(vars[U1], 0));
        this.block1_.setVelocity(new Vector(vars[V1], 0));
        this.block2_.setPosition(new Vector(vars[U2], 0));
        this.block2_.setVelocity(new Vector(vars[V2], 0));
    }
    ;
    startDrag(simObject, _location, _offset, _dragBody, _modifiers) {
        if (simObject == this.block1_) {
            this.dragBlock_ = 0;
            return true;
        }
        else if (simObject == this.block2_) {
            this.dragBlock_ = 1;
            return true;
        }
        this.dragBlock_ = -1;
        return false;
    }
    ;
    mouseDrag(simObject, location, offset) {
        const p = location.subtract(offset);
        if (this.dragBlock_ == 0 || this.dragBlock_ == 1) {
            const block = this.dragBlock_ == 0 ? this.block1_ : this.block2_;
            if (simObject != block) {
                return;
            }
            const va = this.getVarsList();
            va.setValue(this.dragBlock_, p.getX());
            va.setValue(this.dragBlock_ + V1, 0);
            va.incrSequence(KE, PE, TE);
            this.moveObjects(va.getValues());
        }
    }
    ;
    finishDrag(_simObject, _location, _offset) {
        this.dragBlock_ = -1;
    }
    ;
    handleKeyEvent(_evt, _pressed, _modifiers) {
    }
    ;
    evaluate(vars, change, _timeStep) {
        Util.zeroArray(change);
        change[TIME] = 1.0;
        this.moveObjects(vars);
        const u1 = vars[U1];
        const u2 = vars[U2];
        const S1 = u1 < this.wall1_.getPosition().getX() + this.wall1_.getWidth() / 2 ? -1 : 1;
        const S2 = u2 < u1 ? -1 : 1;
        const S3 = u2 > this.wall2_.getPosition().getX() - this.wall2_.getWidth() / 2 ? -1 : 1;
        if (this.dragBlock_ != 0) {
            change[U1] = vars[V1];
            change[V1] = (-S1 * this.spring1_.getStiffness() * this.spring1_.getStretch()
                + S2 * this.spring2_.getStiffness() * this.spring2_.getStretch()
                - this.damping_ * vars[V1]) / this.block1_.getMass();
        }
        if (this.dragBlock_ != 1) {
            change[U2] = vars[V2];
            change[V2] = (-S2 * this.spring2_.getStiffness() * this.spring2_.getStretch()
                + S3 * this.spring3_.getStiffness() * this.spring3_.getStretch()
                - this.damping_ * vars[V2]) / this.block2_.getMass();
        }
        return null;
    }
    ;
    getThirdSpring() {
        return this.thirdSpring_;
    }
    ;
    setThirdSpring(value) {
        if (value != this.thirdSpring_) {
            this.thirdSpring_ = value;
            this.spring3_.setStiffness(value ? this.stiffness_ : 0);
            this.getVarsList().incrSequence(KE, PE, TE);
            this.broadcastParameter(DoubleSpringSim.en.THIRD_SPRING);
        }
    }
    ;
    getMass1() {
        return this.block1_.getMass();
    }
    ;
    setMass1(value) {
        this.block1_.setMass(value);
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(DoubleSpringSim.en.MASS1);
    }
    ;
    getMass2() {
        return this.block2_.getMass();
    }
    ;
    setMass2(value) {
        this.block2_.setMass(value);
        this.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(DoubleSpringSim.en.MASS2);
    }
    ;
    getLength() {
        return this.springs_[0].getRestLength();
    }
    ;
    setLength(value) {
        for (let i = 0; i < this.springs_.length; i++) {
            this.springs_[i].setRestLength(value);
        }
        this.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(DoubleSpringSim.en.LENGTH);
    }
    ;
    getStiffness() {
        return this.stiffness_;
    }
    ;
    setStiffness(value) {
        if (this.stiffness_ != value) {
            this.stiffness_ = value;
            this.spring1_.setStiffness(value);
            this.spring2_.setStiffness(value);
            if (this.thirdSpring_) {
                this.spring3_.setStiffness(value);
            }
            this.getVarsList().incrSequence(PE, TE);
            this.broadcastParameter(DoubleSpringSim.en.STIFFNESS);
        }
    }
    ;
    getDamping() {
        return this.damping_;
    }
    ;
    setDamping(value) {
        this.damping_ = value;
        this.broadcastParameter(DoubleSpringSim.en.DAMPING);
    }
    ;
}
DoubleSpringSim.en = {
    ACCELERATION: 'acceleration',
    DAMPING: 'damping',
    MASS1: 'mass-1',
    MASS2: 'mass-2',
    POSITION: 'position',
    LENGTH: 'spring length',
    STIFFNESS: 'spring stiffness',
    VELOCITY: 'velocity',
    REST_STATE: 'rest state',
    THIRD_SPRING: 'third spring'
};
DoubleSpringSim.de_strings = {
    ACCELERATION: 'Beschleunigung',
    DAMPING: 'Dämpfung',
    MASS1: 'Masse-1',
    MASS2: 'Masse-2',
    POSITION: 'Position',
    LENGTH: 'Federlänge',
    STIFFNESS: 'Federsteifheit',
    VELOCITY: 'Geschwindigkeit',
    REST_STATE: 'ruhe Zustand',
    THIRD_SPRING: 'dritte Feder'
};
DoubleSpringSim.i18n = Util.LOCALE === 'de' ? DoubleSpringSim.de_strings : DoubleSpringSim.en;
Util.defineGlobal('sims$springs$DoubleSpringSim', DoubleSpringSim);
