import { AbstractApp } from '../common/AbstractApp.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { MoleculeSim, KE, PE, TE, U1X, U1Y } from './MoleculeSim.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { PointMass } from '../../lab/model/PointMass.js';
import { RandomLCG } from '../../lab/util/Random.js';
import { SimList } from '../../lab/model/SimList.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Spring } from '../../lab/model/Spring.js';
import { SpringNonLinear1 } from './SpringNonLinear1.js';
import { SpringNonLinear2 } from './SpringNonLinear2.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
;
export class Molecule3App extends AbstractApp {
    constructor(elem_ids, numAtoms) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new MoleculeSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.random_ = new RandomLCG(78597834798);
        this.msm_ = [];
        this.sg_ = [];
        this.nsg_ = [];
        this.springType_ = 1;
        this.mass = 0.5;
        this.redMass = 0.5;
        this.length = 3;
        this.redLength = 3;
        this.stiffness = 6;
        this.redStiffness = 6;
        this.attract = 30;
        this.numAtoms_ = numAtoms;
        this.layout.getSimCanvas().setBackground('black');
        this.protoSpring = new DisplaySpring();
        this.protoSpring.setWidth(0.3);
        this.protoSpring.setColorCompressed('#0c0');
        this.protoSpring.setColorExpanded('green');
        this.protoSpring.setThickness(3);
        this.protoSpecialSpring = new DisplaySpring();
        this.protoSpecialSpring.setWidth(0.3);
        this.protoSpecialSpring.setColorCompressed('#b55');
        this.protoSpecialSpring.setColorExpanded('red');
        this.protoSpecialSpring.setThickness(3);
        Util.assert(this.simList.length() == 1);
        this.simList.addObserver(this);
        this.addBody(this.sim.getWalls());
        this.addPlaybackControls();
        let pn;
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.NUM_ATOMS, Molecule3App.i18n.NUM_ATOMS, () => this.getNumAtoms(), a => this.setNumAtoms(a)));
        this.addControl(new SliderControl(pn, 1, 6, false, 5));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.SPRING_TYPE, Molecule3App.i18n.SPRING_TYPE, () => this.getSpringType(), a => this.setSpringType(a), [Molecule3App.i18n.LINEAR,
            Molecule3App.i18n.NON_LINEAR,
            Molecule3App.i18n.PSEUDO_GRAVITY], [1,
            2,
            3]));
        this.addControl(new ChoiceControl(pn));
        pn = this.sim.getParameterNumber(MoleculeSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = this.sim.getParameterNumber(MoleculeSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 1, false));
        pn = this.sim.getParameterNumber(MoleculeSim.en.ELASTICITY);
        this.addControl(new SliderControl(pn, 0, 1, false));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.MASS, Molecule3App.i18n.MASS, () => this.getMass(), a => this.setMass(a)));
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.MASS_SPECIAL, Molecule3App.i18n.MASS_SPECIAL, () => this.getMassSpecial(), a => this.setMassSpecial(a)));
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.LENGTH, Molecule3App.i18n.LENGTH, () => this.getLength(), a => this.setLength(a)));
        this.addControl(this.controlLength = new SliderControl(pn, 0.1, 10.1, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.LENGTH_SPECIAL, Molecule3App.i18n.LENGTH_SPECIAL, () => this.getLengthSpecial(), a => this.setLengthSpecial(a)));
        this.addControl(this.controlLengthSpecial = new SliderControl(pn, 0.1, 10.1, true));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.STIFFNESS, Molecule3App.i18n.STIFFNESS, () => this.getStiffness(), a => this.setStiffness(a)));
        this.addControl(new SliderControl(pn, 0, 100, false));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.STIFFNESS_SPECIAL, Molecule3App.i18n.STIFFNESS_SPECIAL, () => this.getStiffnessSpecial(), a => this.setStiffnessSpecial(a)));
        this.addControl(new SliderControl(pn, 0, 100, false));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.ATTRACT_FORCE, Molecule3App.i18n.ATTRACT_FORCE, () => this.getAttractForce(), a => this.setAttractForce(a)));
        this.addControl(this.controlAttract = new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, Molecule3App.en.WALL_SIZE, Molecule3App.i18n.WALL_SIZE, () => this.getWallSize(), a => this.setWallSize(a)));
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.config();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.graph.line.setXVariable(U1X);
        this.graph.line.setYVariable(U1Y);
        this.timeGraph.line1.setYVariable(KE);
        this.timeGraph.line2.setYVariable(PE);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + super.toString();
    }
    ;
    getClassName() {
        return 'Molecule3App';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('protoSpecialSpring|protoSpring', myName + '.');
    }
    ;
    addBody(obj) {
        if (this.displayList.find(obj)) {
            return;
        }
        if (obj instanceof PointMass) {
            const pm = obj;
            if (pm.getName().match(/^WALL/)) {
                const walls = new DisplayShape(pm);
                walls.setFillStyle('');
                walls.setStrokeStyle('gray');
                this.displayList.add(walls);
            }
            else {
                let cm = 'black';
                switch (pm.getName()) {
                    case 'ATOM1':
                        cm = 'red';
                        break;
                    case 'ATOM2':
                        cm = 'blue';
                        break;
                    case 'ATOM3':
                        cm = 'magenta';
                        break;
                    case 'ATOM4':
                        cm = 'orange';
                        break;
                    case 'ATOM5':
                        cm = 'gray';
                        break;
                    case 'ATOM6':
                        cm = 'green';
                        break;
                    default: cm = 'pink';
                }
                const atom = new DisplayShape(pm);
                atom.setFillStyle(cm);
                this.displayList.add(atom);
            }
        }
        else if (obj instanceof Spring) {
            const s = obj;
            const proto = s.getName().match(/^SPECIAL/) ? this.protoSpecialSpring :
                this.protoSpring;
            this.displayList.add(new DisplaySpring(s, proto));
        }
    }
    ;
    observe(event) {
        if (event.getSubject() == this.simList) {
            const obj = event.getValue();
            if (event.nameEquals(SimList.OBJECT_ADDED)) {
                this.addBody(obj);
            }
            else if (event.nameEquals(SimList.OBJECT_REMOVED)) {
                const d = this.displayList.find(obj);
                if (d) {
                    this.displayList.remove(d);
                }
            }
        }
    }
    ;
    config() {
        const numAtoms = this.numAtoms_;
        if (numAtoms < 1 || numAtoms > 6) {
            throw 'too many atoms ' + numAtoms;
        }
        this.sim.cleanSlate();
        this.msm_ = Molecule3App.getMSM(numAtoms);
        this.sg_ = Molecule3App.getSG(numAtoms);
        this.nsg_ = Molecule3App.getNSG(this.msm_.length, this.sg_);
        for (let i = 0; i < numAtoms; i++) {
            const atom = PointMass.makeCircle(0.5, 'atom' + (i + 1));
            atom.setMass(i == 0 ? this.redMass : this.mass);
            this.sim.addAtom(atom);
        }
        const atoms = this.sim.getAtoms();
        for (let i = 0; i < this.msm_.length; i++) {
            const special = this.sg_.includes(i);
            const name = (special ? 'special ' : '') + 'spring ' + i;
            const atom1 = atoms[this.msm_[i][0]];
            const atom2 = atoms[this.msm_[i][1]];
            const loc = Vector.ORIGIN;
            const len = special ? this.redLength : this.length;
            const stiff = special ? this.redStiffness : this.stiffness;
            let spring;
            switch (this.springType_) {
                case 1:
                    spring = new Spring(name, atom1, loc, atom2, loc, len, stiff);
                    break;
                case 2:
                    spring = new SpringNonLinear1(name, atom1, loc, atom2, loc, len, stiff);
                    break;
                case 3:
                    spring = new SpringNonLinear2(name, atom1, loc, atom2, loc, len, stiff, this.attract);
                    break;
                default:
                    throw '';
            }
            const linearSpring = this.springType_ == 1;
            this.controlLength.setEnabled(linearSpring);
            this.controlLengthSpecial.setEnabled(linearSpring);
            this.controlAttract.setEnabled(this.springType_ == 3);
            spring.setDamping(0);
            this.sim.addSpring(spring);
        }
        this.initialPositions(numAtoms);
        this.sim.saveInitialState();
        this.sim.modifyObjects();
        if (this.easyScript) {
            this.easyScript.update();
        }
    }
    ;
    static getMSM(numAtoms) {
        switch (numAtoms) {
            case 1: return [];
            case 2: return [[0, 1]];
            case 3: return [[0, 1], [1, 2], [2, 0]];
            case 4: return [[0, 1], [1, 2], [2, 3], [3, 0], [1, 3], [0, 2]];
            case 5: return [[0, 1], [1, 2], [2, 3], [3, 4], [4, 0], [4, 2], [4, 1],
                [0, 3], [1, 3], [0, 2]];
            case 6: return [[0, 1], [1, 2], [2, 3], [3, 4], [4, 5], [5, 0], [0, 2], [2, 4],
                [4, 0], [1, 3], [3, 5], [5, 1], [0, 3], [1, 4], [2, 5]];
        }
        throw '';
    }
    ;
    static getSG(numAtoms) {
        switch (numAtoms) {
            case 1: return [];
            case 2: return [];
            case 3: return [0];
            case 4: return [0, 3, 5];
            case 5: return [0, 4, 7, 9];
            case 6: return [12, 13, 14];
        }
        throw '';
    }
    ;
    static getNSG(num_springs, sg) {
        const nsg = [];
        for (let i = 0; i < num_springs; i++) {
            if (!sg.includes(i)) {
                nsg.push(i);
            }
        }
        return nsg;
    }
    ;
    initialPositions(numAtoms) {
        const vars = this.sim.getVarsList().getValues();
        const r = 1.0;
        for (let i = 0; i < numAtoms; i++) {
            const idx = 4 * i;
            vars[U1X + idx] = r * Math.cos(i * 2 * Math.PI / numAtoms);
            vars[U1Y + idx] = r * Math.sin(i * 2 * Math.PI / numAtoms);
        }
        this.sim.getVarsList().setValues(vars);
    }
    ;
    getNumAtoms() {
        return this.numAtoms_;
    }
    ;
    setNumAtoms(value) {
        if (value < 1 || value > 6) {
            throw 'too many atoms ' + value;
        }
        this.numAtoms_ = value;
        this.config();
        this.sim.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(Molecule3App.en.NUM_ATOMS);
    }
    ;
    getSpringType() {
        return this.springType_;
    }
    ;
    setSpringType(value) {
        if (this.springType_ != value) {
            this.springType_ = value;
            this.config();
            this.sim.getVarsList().incrSequence(KE, PE, TE);
            this.broadcastParameter(Molecule3App.en.SPRING_TYPE);
        }
    }
    ;
    getMass() {
        return this.mass;
    }
    ;
    setMass(value) {
        this.mass = value;
        this.sim.getAtoms().forEach((atom, idx) => {
            if (idx > 0) {
                atom.setMass(value);
            }
        });
        this.sim.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(Molecule3App.en.MASS);
    }
    ;
    getMassSpecial() {
        return this.redMass;
    }
    ;
    setMassSpecial(value) {
        this.redMass = value;
        this.sim.getAtoms()[0].setMass(value);
        this.sim.getVarsList().incrSequence(KE, PE, TE);
        this.broadcastParameter(Molecule3App.en.MASS_SPECIAL);
    }
    ;
    getLength() {
        return this.length;
    }
    ;
    setLength(value) {
        this.length = value;
        for (let i = 0; i < this.nsg_.length; i++) {
            this.sim.getSprings()[this.nsg_[i]].setRestLength(value);
        }
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule3App.en.LENGTH);
    }
    ;
    getLengthSpecial() {
        return this.redLength;
    }
    ;
    setLengthSpecial(value) {
        this.redLength = value;
        for (let i = 0; i < this.sg_.length; i++) {
            this.sim.getSprings()[this.sg_[i]].setRestLength(value);
        }
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule3App.en.LENGTH_SPECIAL);
    }
    ;
    getStiffness() {
        return this.stiffness;
    }
    ;
    setStiffness(value) {
        this.stiffness = value;
        for (let i = 0; i < this.nsg_.length; i++) {
            this.sim.getSprings()[this.nsg_[i]].setStiffness(value);
        }
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule3App.en.STIFFNESS);
    }
    ;
    getStiffnessSpecial() {
        return this.redStiffness;
    }
    ;
    setStiffnessSpecial(value) {
        this.redStiffness = value;
        for (let i = 0; i < this.sg_.length; i++) {
            this.sim.getSprings()[this.sg_[i]].setStiffness(value);
        }
        this.sim.getVarsList().incrSequence(PE, TE);
        this.broadcastParameter(Molecule3App.en.STIFFNESS_SPECIAL);
    }
    ;
    getAttractForce() {
        return this.attract;
    }
    ;
    setAttractForce(value) {
        if (this.attract != value) {
            this.attract = value;
            this.sim.getSprings().forEach(spr => {
                if (spr instanceof SpringNonLinear2) {
                    spr.setAttract(value);
                }
            });
            this.sim.getVarsList().incrSequence(PE, TE);
            this.broadcastParameter(Molecule3App.en.ATTRACT_FORCE);
        }
    }
    ;
    getWallSize() {
        return this.sim.getWalls().getWidth();
    }
    ;
    setWallSize(value) {
        const w = this.sim.getWalls();
        w.setWidth(value);
        w.setHeight(value);
        let simRect = w.getBoundsWorld();
        const max = 60;
        simRect = simRect.intersection(new DoubleRect(-max, -max, max, max));
        this.simView.setSimRect(simRect);
        this.broadcastParameter(Molecule3App.en.WALL_SIZE);
    }
    ;
}
Molecule3App.en = {
    MASS: 'mass',
    MASS_SPECIAL: 'red mass',
    LENGTH: 'spring length',
    LENGTH_SPECIAL: 'red spring length',
    STIFFNESS: 'spring stiffness',
    STIFFNESS_SPECIAL: 'red spring stiffness',
    ATTRACT_FORCE: 'attract force',
    NUM_ATOMS: 'number of atoms',
    LINEAR: 'linear',
    NON_LINEAR: 'non-linear',
    PSEUDO_GRAVITY: 'pseudo-gravity',
    SPRING_TYPE: 'spring type',
    WALL_SIZE: 'wall size',
};
Molecule3App.de_strings = {
    MASS: 'Masse',
    MASS_SPECIAL: 'rote Masse',
    LENGTH: 'Federlänge',
    LENGTH_SPECIAL: 'rote Federlänge',
    STIFFNESS: 'Federsteifheit',
    STIFFNESS_SPECIAL: 'rote Federsteifheit',
    ATTRACT_FORCE: 'anziehen Kraft',
    NUM_ATOMS: 'zahl Masse',
    LINEAR: 'linear',
    NON_LINEAR: 'nichtlinear',
    PSEUDO_GRAVITY: 'pseudo-Gravitation',
    SPRING_TYPE: 'FederTyp',
    WALL_SIZE: 'Wand Größe',
};
Molecule3App.i18n = Util.LOCALE === 'de' ? Molecule3App.de_strings : Molecule3App.en;
Util.defineGlobal('sims$springs$Molecule3App', Molecule3App);
