import { AbstractApp } from '../common/AbstractApp.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { FunctionVariable } from '../../lab/model/FunctionVariable.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SingleSpringSim } from './SingleSpringSim.js';
import { TabLayout3 } from '../common/TabLayout3.js';
import { Util } from '../../lab/util/Util.js';
export class SingleSpring3App extends AbstractApp {
    constructor(elem_ids, opt_name) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-3, -2, 3, 2);
        const sim = new SingleSpringSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim, opt_name);
        this.block = new DisplayShape(this.simList.getPointMass('block'));
        this.block.setFillStyle('blue');
        this.displayList.add(this.block);
        this.spring = new DisplaySpring(this.simList.getSpring('spring'));
        this.spring.setWidth(0.4);
        this.spring.setThickness(6);
        this.displayList.add(this.spring);
        const va = sim.getVarsList();
        va.addVariable(new FunctionVariable(va, 'sin_time', 'sin(time)', () => Math.sin(sim.getTime())));
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(SingleSpringSim.en.MASS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(SingleSpringSim.en.DAMPING);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(SingleSpringSim.en.SPRING_STIFFNESS);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(SingleSpringSim.en.SPRING_LENGTH);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(SingleSpringSim.en.FIXED_POINT);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', block: ' + this.block.toStringShort()
            + ', spring: ' + this.spring.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'SingleSpring3App';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('block|spring', myName + '.');
    }
    ;
    makeLayout(elem_ids) {
        const canvasWidth = 800;
        const canvasHeight = Math.round(canvasWidth * this.simRect.getHeight() /
            this.simRect.getWidth());
        return new TabLayout3(elem_ids, canvasWidth, canvasHeight);
    }
    ;
}
Util.defineGlobal('sims$springs$SingleSpring3App', SingleSpring3App);
