import { Force } from '../../lab/model/Force.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class SpringNonLinear2 extends Spring {
    constructor(name, body1, attach1_body, body2, attach2_body, restLength, stiffness = 8, attract = 6) {
        super(name, body1, attach1_body, body2, attach2_body, restLength, stiffness, false);
        this.minPE_ = 0;
        this.attract_ = attract;
        this.calcMinPE();
    }
    ;
    getClassName() {
        return 'SpringNonLinear2';
    }
    ;
    calculateForces() {
        const point1 = this.getStartPoint();
        const point2 = this.getEndPoint();
        const v = point2.subtract(point1);
        const len = v.length();
        const body1 = this.getBody1();
        const body2 = this.getBody2();
        const m1 = body1.getMass();
        const m2 = body2.getMass();
        const S = this.getStiffness();
        const G = this.attract_;
        const sf = G * m1 * m2 / (len * len) - S * Math.pow(len, -3);
        const fx = sf * (v.getX() / len);
        const fy = sf * (v.getY() / len);
        let f = new Vector(fx, fy, 0);
        if (this.getDamping() != 0) {
            const v1 = body1.getVelocity(this.getAttach1());
            const v2 = body2.getVelocity(this.getAttach2());
            const df = v1.subtract(v2).multiply(-this.getDamping());
            f = f.add(df);
        }
        return [new Force('spring', body1, point1, 1, f, 1),
            new Force('spring', body2, point2, 1, f.multiply(-1), 1)];
    }
    ;
    getPotentialEnergy() {
        const len = this.getLength();
        return this.potentialEnergy(len) - this.minPE_;
    }
    ;
    potentialEnergy(len) {
        const m1 = this.getBody1().getMass();
        const m2 = this.getBody2().getMass();
        const S = this.getStiffness();
        const G = this.attract_;
        return -G * m1 * m2 / len + (S / 2) / (len * len);
    }
    ;
    minPELen() {
        const m1 = this.getBody1().getMass();
        const m2 = this.getBody2().getMass();
        const S = this.getStiffness();
        const G = this.attract_;
        return S / (G * m1 * m2);
    }
    ;
    calcMinPE() {
        this.minPE_ = this.potentialEnergy(this.minPELen());
    }
    ;
    setStiffness(stiffness) {
        super.setStiffness(stiffness);
        this.calcMinPE();
    }
    ;
    getAttract() {
        return this.attract_;
    }
    ;
    setAttract(attract) {
        this.attract_ = attract;
        this.calcMinPE();
        this.setChanged();
    }
    ;
}
Util.defineGlobal('sims$springs$SpringNonLinear2', SpringNonLinear2);
