import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ConstantForceLaw } from '../lab/model/ConstantForceLaw.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { DoNothingApp } from '../sims/engine2D/DoNothingApp.js';
import { DoubleRect } from '../lab/util/DoubleRect.js';
import { Force } from '../lab/model/Force.js';
import { RotatingTestForce } from '../sims/engine2D/RotatingTestForce.js';
import { RungeKutta } from '../lab/model/RungeKutta.js';
import { Vector } from '../lab/util/Vector.js';
import { schedule, setTestName, assertLessThan } from './TestRig.js';
import { makeVars, setBodyVars, runTest, runExceptionTest, checkTightJoints } from './Engine2DTestRig.js';
const groupName = 'DoNothingTest.';
export class DoNothingTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(DoNothingTest.do_nothing_grinder_test1);
        schedule(DoNothingTest.do_nothing_grinder_test1b);
        schedule(DoNothingTest.do_nothing_grinder_test2);
        schedule(DoNothingTest.do_nothing_variable_test);
        schedule(DoNothingTest.do_nothing_error);
    }
    ;
    static do_nothing_variable_setup(sim, advance) {
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setDistanceTol(0.01);
        sim.setVelocityTol(0.5);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setShowForces(false);
        sim.setCollisionAccuracy(0.6);
        advance.setDiffEqSolver(new RungeKutta(sim));
        sim.setExtraAccel("velocity");
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        DoNothingApp.setup(sim, true);
        sim.setElasticity(0.8);
        sim.setSimRect(new DoubleRect(-5, -5, 5, 5));
        const handle = sim.getBody('handle');
        sim.addForceLaw(new RotatingTestForce(sim, handle, new Vector(0, -3), 2, 0.3));
    }
    ;
    static do_nothing_variable_test() {
        setTestName(groupName + 'do_nothing_variable_test');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        DoNothingTest.do_nothing_variable_setup(sim, advance);
        const vars = makeVars((4 + 3) * 6);
        setBodyVars(sim, vars, 0, -2.350918, -1.4252083, 0.2181222, -0.3159406, -7.2797567, -0.9370725);
        setBodyVars(sim, vars, 1, 0, -0, 1.7390377, -2.5189213, 3.1415927, -0);
        setBodyVars(sim, vars, 2, -2.688075, -1.6296046, -0, 0, 1.5707963, 0);
        setBodyVars(sim, vars, 3, 2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 4, 2.507, 0, -2.507, 0, 0, 0);
        setBodyVars(sim, vars, 5, -2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 6, -2.507, 0, -2.507, 0, 0, 0);
        runTest(sim, advance, 40.0, vars, 0.00001);
        checkTightJoints(sim, 0.005);
    }
    ;
    static do_nothing_constant_setup(sim, advance) {
        sim.setDistanceTol(0.01);
        sim.setVelocityTol(0.5);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setShowForces(false);
        sim.setCollisionAccuracy(0.6);
        sim.setExtraAccel("velocity");
        advance.setJointSmallImpacts(true);
        advance.setDiffEqSolver(new RungeKutta(sim));
        advance.setTimeStep(0.025);
        DoNothingApp.setup(sim, true);
        sim.setElasticity(0.8);
        sim.setSimRect(new DoubleRect(-5, -5, 5, 5));
        const handle = sim.getBody('handle');
        const f = new Force('turning', handle, new Vector(0, -3), 0, Vector.EAST, 0);
        sim.addForceLaw(new ConstantForceLaw(f));
    }
    ;
    static do_nothing_grinder_test1() {
        setTestName(groupName + 'do_nothing_grinder_test1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        DoNothingTest.do_nothing_constant_setup(sim, advance);
        const vars = makeVars((4 + 3) * 6);
        setBodyVars(sim, vars, 0, -2.7257349, -4.1971916, -0.0918363, 2.5622131, 61.0303112, 6.5543294);
        setBodyVars(sim, vars, 1, 0, -0, -0.7321898, 20.4279338, 3.1415927, -0);
        setBodyVars(sim, vars, 2, -3.1166463, -4.7991321, 0, -0, 1.5707963, -0);
        setBodyVars(sim, vars, 3, 2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 4, 2.507, 0, -2.507, 0, 0, 0);
        setBodyVars(sim, vars, 5, -2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 6, -2.507, 0, -2.507, 0, 0, 0);
        runTest(sim, advance, 20.0, vars, 0.00001);
        checkTightJoints(sim, 0.005);
        assertLessThan(sim.getMaxForce(), 200);
    }
    ;
    static do_nothing_constant_1b_setup(sim, advance) {
        DoNothingTest.do_nothing_constant_setup(sim, advance);
        advance.setJointSmallImpacts(false);
        sim.setExtraAccel("none");
    }
    ;
    static do_nothing_grinder_test1b() {
        setTestName(groupName + 'do_nothing_grinder_test1b');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        DoNothingTest.do_nothing_constant_1b_setup(sim, advance);
        const vars = makeVars((4 + 3) * 6);
        setBodyVars(sim, vars, 0, -2.7245937, -4.2309373, -0.0925957, 2.560584, 61.028377, 6.5529456);
        setBodyVars(sim, vars, 1, -0, -0, -0.7382445, 20.4149451, 3.1415927, 0);
        setBodyVars(sim, vars, 2, -3.1153414, -4.8377173, -0, -0, 1.5707963, 0);
        setBodyVars(sim, vars, 3, 2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 4, 2.507, 0, -2.507, 0, 0, 0);
        setBodyVars(sim, vars, 5, -2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 6, -2.507, 0, -2.507, 0, 0, 0);
        runTest(sim, advance, 20.0, vars, 0.00001);
        checkTightJoints(sim, 0.01);
    }
    ;
    static do_nothing_grinder_test2() {
        setTestName(groupName + 'do_nothing_grinder_test2');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        DoNothingTest.do_nothing_constant_setup(sim, advance);
        runTest(sim, advance, 10);
        const fls = sim.getForceLaws();
        for (let i = 0; i < fls.length; i++) {
            sim.removeForceLaw(fls[i]);
        }
        const vars = makeVars((4 + 3) * 6);
        setBodyVars(sim, vars, 0, 1.8024594, 6.5378506, 0.307294, -0.7887431, 107.5135207, 3.0512303);
        setBodyVars(sim, vars, 1, 0, 0, 2.4499844, -6.2884668, 3.1415927, 0);
        setBodyVars(sim, vars, 2, 2.0609592, 7.4754767, 0, 0, 1.5707963, 0);
        setBodyVars(sim, vars, 3, 2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 4, 2.507, 0, -2.507, 0, 0, 0);
        setBodyVars(sim, vars, 5, -2.507, 0, 2.507, 0, 0, 0);
        setBodyVars(sim, vars, 6, -2.507, 0, -2.507, 0, 0, 0);
        runTest(sim, advance, 40.0, vars, 0.00001, 0.0, 0.001, -1);
        checkTightJoints(sim, 0.005);
    }
    ;
    static do_nothing_error() {
        setTestName(groupName + 'do_nothing_error');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        DoNothingTest.do_nothing_variable_setup(sim, advance);
        sim.setRandomSeed(9872392);
        sim.setExtraAccel("none");
        advance.setJointSmallImpacts(false);
        runExceptionTest(advance, 100);
    }
    ;
}
